/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.camera;


import com.ml.vision.autoface.detection.FaceDetection;
import com.ml.vision.autoface.detection.objects.Yolov3;
import com.ml.vision.autoface.listener.CameraListener;
import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.cvFlip;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.Frame;
import org.bytedeco.javacv.FrameGrabber;
import org.bytedeco.javacv.OpenCVFrameConverter;

/**
 *
 * @author apple
 * Reference for DVR https://www.securitycameraking.com/securityinfo/how-to-back-up-your-dvr-to-an-ftp-server/
 * Reference for IP camera https://www.ispyconnect.com/man.aspx?n=Hikvision
 */
public class CameraAdvanced extends Thread implements Runnable{
    
    boolean log = true;
    
    private long camId;// database ID
    private String camName;// brand name or other
    private int usbIndex;// for USB connected camera
    private String ipAddress;// for IP camera
    private int buildingLevel;// camera mounted level
    private String building;// camera mounted level
    private String location;// geographic location for enterprise
    private String description;// 
    private String position;// entrance, exit, leftside, rightside, general
    private String captureMode;// primary for register, secondary for monitoring
    private String gateNumber;// entrance number, gate number, counter number
    private boolean active;//
    private int condition;
    private boolean alert;
    private boolean counterOn;
    private int arrayIndex;
    
    private int algorithm = FaceDetection.ALGORITHM_CASCADE_CLASSIFIER;
    private long delay = 2;
    private int framePerSecond = 2;
    private String[] data;
    private boolean willStop = false;
    private long lastTime = 1;
    private long startOn = 0;
    private long lastFound = 0;
    private int lostCount = Integer.MAX_VALUE;
    
    private opencv_core.Mat lastCapture;
    private FrameGrabber grabber;
    private OpenCVFrameConverter.ToMat converter = new OpenCVFrameConverter.ToMat();
    private CameraListener listener;
    private FaceDetection detection;
    private Yolov3 objectDetector;
    private boolean rectangleFace = true;
    private boolean sortFaceBySize = true;
    
    public CameraAdvanced( ){ }

    public CameraAdvanced( String[] data ){
        
        this.data = data;
        this.camId = Long.parseLong(data[0]);
        this.camName = data[1];
        this.usbIndex = Integer.parseInt(data[2]);
        this.ipAddress = data[3];
        this.buildingLevel = Integer.parseInt(data[4]);
        this.building = data[5];
        this.location = data[6];
        this.position = data[7];
        this.captureMode = data[8];
        this.gateNumber = data[9];
        this.description = data[11];
        this.active = Boolean.parseBoolean(data[12]);
        this.alert = Boolean.parseBoolean(data[13]);
        this.counterOn = Boolean.parseBoolean(data[14]);
        
        try {
            if( data[3] != null && !data[3].isEmpty()){
                grabber = new FFmpegFrameGrabber( ipAddress );
                grabber.setOption( "rtsp_transport", "tcp" );
                grabber.setImageHeight( 720 );
                grabber.setImageWidth( 1280 );
            }else
                grabber = FrameGrabber.createDefault( usbIndex );
            
        } catch ( Exception ex) {
            willStop = true;
            System.out.println("Camera failed");
            //ex.printStackTrace();
            if( log )
                Logger.getLogger(CameraAdvanced.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    public CameraAdvanced( int index ){
        try{
            grabber = FrameGrabber.createDefault( index );
        }catch(Exception ex){
            willStop = true;
            if( log ) ex.printStackTrace();
        }
    }
    
    public CameraAdvanced( String ipCameraAddress ){
        try{
            grabber = new FFmpegFrameGrabber( ipCameraAddress );
            grabber.setOption( "rtsp_transport", "tcp" );
            grabber.setImageHeight( 720 );
            grabber.setImageWidth( 1280 );
//            start();
        }catch(Exception ex){
            willStop = true;
            if( log ) ex.printStackTrace();
        }
    }
       
    public CameraAdvanced( File f ){
        try{
            grabber = FFmpegFrameGrabber.createDefault( f );
//            start();
        }catch(Exception ex){
            willStop = true;
            if( log ) ex.printStackTrace();
        }
    }
   
    @Override
    public void run(){
        try{
            if( grabber != null )
                grabber.start();
            else{
                listener.onFailed(this);
                return;
            }
            
            lastTime = System.currentTimeMillis();
            setStartOn( System.currentTimeMillis() );
            setLastFound(System.currentTimeMillis());
            
            
            while( true ){
                if( willStop ) break;
                
                setLostCount( Integer.MAX_VALUE );
                
                if( grabber == null ) {
                    listener.onFailed(this);
                    continue;
                }
                try{
                    // flipping image
                    Frame tmpFrame = grabber.grab();
                    if( tmpFrame == null ) continue;
                    
                    opencv_core.IplImage frame = converter.convertToIplImage( tmpFrame );
                    cvFlip( frame, frame, +1 );
                    lastCapture = converter.convert( converter.convert( frame ) );
                    
                    if( lastCapture == null ) {
                        listener.onFailed(this);
                        continue;
                    }
                                            
                    if( System.currentTimeMillis() - lastTime >= 1000 / getFramePerSecond() ){
                        lastTime = System.currentTimeMillis();
                        
                        if( detection != null ){
                            listener.onCapture( detection.detectFace( lastCapture, true, isRectangleFace(), getAlgorithm(), isSortFaceBySize() ), this );
                        }
                        if( objectDetector != null ){
                            //objectDetector.detect( lastCapture );
                            //listener.onDemographic( objectDetector.getDemographic(), objectDetector.getIncreasedCount() );
                        }
                        
                        //listener.onCaptureByFPS( this );
                        listener.onCapture( this );
                    }
                    setLastFound(System.currentTimeMillis());
                    setLostCount( 0 );
                    //setDelay( 2 );
                }catch( Exception ex){if(log)ex.printStackTrace();}
                
                sleep( getDelay() );
            }
        }catch(Exception ex){
            if(log)ex.printStackTrace();
            try{
                if( grabber != null) grabber.restart();    
            }catch(Exception x){if(log)x.printStackTrace(); }
            
        }
    }
    
    public void close(){
        willStop = true;
        try {
            if( grabber != null ){
                grabber.release( );
                grabber.stop( );
            }
            stop();
        } catch (FrameGrabber.Exception ex) {
            if( log )
                Logger.getLogger(CameraAdvanced.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    public FrameGrabber getGrabber(){
        return grabber;
    }

    public int getAlgorithm() {
        return algorithm;
    }

    public void setAlgorithm(int algorithm) {
        this.algorithm = algorithm;
    }
    
    public void setListener( CameraListener listener) {
        this.listener = listener;
    }

    public long getDelay() {
        return delay;
    }

    public void setDelay(long delay) {
        this.delay = delay;
    }

    public int getUsbIndex() {
        return usbIndex;
    }

    public void setUsbIndex(int usbIndex) {
        this.usbIndex = usbIndex;
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public int getBuildingLevel() {
        return buildingLevel;
    }

    public void setBuildingLevel(int buildingLevel) {
        this.buildingLevel = buildingLevel;
    }

    public String getBuilding() {
        return building;
    }

    public void setBuilding(String building) {
        this.building = building;
    }

    public String getLocation() {
        return location;
    }

    public void setLocation(String location) {
        this.location = location;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getPosition() {
        return position;
    }

    public void setPosition(String position) {
        this.position = position;
    }

    public String getCaptureMode() {
        return captureMode;
    }

    public void setCaptureMode(String captureMode) {
        this.captureMode = captureMode;
    }

    public String getGateNumber() {
        return gateNumber;
    }

    public void setGateNumber(String gateNumber) {
        this.gateNumber = gateNumber;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public int getCondition() {
        return condition;
    }

    public void setCondition(int condition) {
        this.condition = condition;
    }

    public long getCamId() {
        return camId;
    }

    public void setCamId(long camId) {
        this.camId = camId;
    }

    public String getCamName() {
        return camName;
    }

    public void setCamName(String camName) {
        this.camName = camName;
    }

    public opencv_core.Mat getLastCapture() {
        return lastCapture;
    }

    public void setLastCapture(opencv_core.Mat lastCapture) {
        this.lastCapture = lastCapture;
    }

    public int getFramePerSecond() {
        return framePerSecond;
    }

    public void setFramePerSecond(int framePerSecond) {
        this.framePerSecond = framePerSecond;
    }

    public FaceDetection getDetection() {
        return detection;
    }

    public void setDetection(FaceDetection detection) {
        this.detection = detection;
    }

    public Yolov3 getObjectDetector() {
        return objectDetector;
    }

    public void setObjectDetector(Yolov3 objectDetector) {
        this.objectDetector = objectDetector;
    }

    public boolean isRectangleFace() {
        return rectangleFace;
    }

    public void setRectangleFace(boolean rectangleFace) {
        this.rectangleFace = rectangleFace;
    }

    public boolean isSortFaceBySize() {
        return sortFaceBySize;
    }

    public void setSortFaceBySize(boolean sortFaceBySize) {
        this.sortFaceBySize = sortFaceBySize;
    }

    public int getArrayIndex() {
        return arrayIndex;
    }

    public void setArrayIndex(int arrayIndex) {
        this.arrayIndex = arrayIndex;
    }

    public boolean isAlert() {
        return alert;
    }

    public void setAlert(boolean alert) {
        this.alert = alert;
    }

    public boolean isCounterOn() {
        return counterOn;
    }

    public void setCounterOn(boolean counterOn) {
        this.counterOn = counterOn;
    }

    public long getStartOn() {
        return startOn;
    }

    public void setStartOn(long startOn) {
        this.startOn = startOn;
    }

    public long getLastFound() {
        return lastFound;
    }

    public void setLastFound(long lastFound) {
        this.lastFound = lastFound;
    }

    public int getLostCount() {
        return lostCount;
    }

    public void setLostCount(int lostCount) {
        this.lostCount = lostCount;
    }

    public String[] getData() {
        return data;
    }

    public void setData(String[] data) {
        this.data = data;
    }
    
    
}
