/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.camera;


import com.ml.vision.autoface.detection.DetectionManager;
import com.ml.vision.autoface.detection.objects.ObjectDetectionManager;
import com.ml.vision.autoface.detection.objects.ObjectDetectionManager.ObjectDetectionManagerListener;
import com.ml.vision.autoface.listener.CameraListener;
import com.ml.vision.autoface.listener.DetectionListener;
import com.ml.vision.autoface.listener.FaceListener;
import java.awt.image.BufferedImage;
import java.util.List;
import java.util.Map;
import org.bytedeco.javacpp.opencv_core;
import org.bytedeco.javacv.OpenCVFrameConverter;
import org.bytedeco.javacv.Java2DFrameConverter;


/**
 *
 * @author apple
 */
public class CameraManager {
    
    private FaceListener listener;
    private OpenCVFrameConverter.ToIplImage converter;
    private java.util.List<CameraAdvanced> cameras;
    private DetectionManager detectionManager;
    private ObjectDetectionManager objectDetectionManager;
    
    public CameraManager( java.util.List<CameraAdvanced> cameras, DetectionManager detectionManager,ObjectDetectionManager objectDetectionManager ){
        this.cameras = cameras;
        this.detectionManager = detectionManager;
        this.objectDetectionManager = objectDetectionManager;
        converter = new OpenCVFrameConverter.ToIplImage();
    }
    
    public CameraManager(DetectionManager detectionManager){
        
        this.detectionManager = detectionManager;
        cameras = new java.util.ArrayList<>();
                
        try{
            java.io.File fv = new java.io.File("/Users/apple/Desktop/Tunnel.2016-HD.mp4");
            CameraAdvanced cam3 = new CameraAdvanced( "http://oviso.axiscam.net/axis-cgi/mjpg/video.cgi" );
            cameras.add( cam3 );
        }catch(Exception ex){ex.printStackTrace();}
        
//        CameraAdvanced cam1 = new CameraAdvanced( 0 );
//        CameraAdvanced cam2 = new CameraAdvanced( 1 );
//        cam1.setAlgorithm(Detection.ALGORITHM_CAFFE_MODEL);
//        cam2.setAlgorithm(Detection.ALGORITHM_CAFFE_MODEL);
//        cameras.add( cam1 );
//        cameras.add( cam2 );
        converter = new OpenCVFrameConverter.ToIplImage();
        
    }
    
    public void start(){
        cameraListenAll();
        if( detectionManager != null ){
            detectorListen();    
            detectionManager.start();
        }
        
        if( objectDetectionManager != null ){
            objectDetectorListen();    
            objectDetectionManager.start();
        }
    }
    
    private void detectorListen(){
        detectionManager.setListener(new DetectionListener() {
            @Override
            public void onDetectedFaces( List<opencv_core.Mat> faces, CameraAdvanced camera, long timeMilis ) {
                //System.out.println("Manager detected");
                listener.onDetectedFaces( faces , camera, camera.getArrayIndex( ) );
            }
        });
    }
    
    private void objectDetectorListen(){
        objectDetectionManager.setListener(new ObjectDetectionManagerListener(){
            @Override
            public void onDemographic(List<String[]> demographic, Map<String, Integer> increasedCount, CameraAdvanced camera, long timeMilis) {
                listener.onDemographic(demographic, increasedCount, camera);
            }
        });
    }
    
    private void cameraListenAll(){
        for( int i = 0; i < cameras.size(); i++ ){
            cameraListen( i );
        }
    }
    
    public void cameraListen( int camindex ){
        try{
            cameras.get( camindex ).setListener(new CameraListener() {
                
                @Override
                public void onFailed(CameraAdvanced camera ) {
                    listener.onFailed( camera, camera.getArrayIndex());
                }

                @Override
                public void onCapture(List<opencv_core.Mat> faces, CameraAdvanced camera) {
                    listener.onDetectedFaces( faces , camera, camera.getArrayIndex() );
                }

                @Override
                public void onCapture(CameraAdvanced camera) {
                    if( camera.getLastCapture() == null ) {
                        listener.onFailed( camera, camera.getArrayIndex());
                        return;
                    }
                    listener.onCapture( camera, camera.getLastCapture(), camera.getArrayIndex() );
                    BufferedImage bimg = new Java2DFrameConverter( ).convert( converter.convert(camera.getLastCapture()) );
                    listener.onCapture( camera, bimg, camera.getArrayIndex() );
                    detectionManager.detect(new Object[]{camera, System.nanoTime()});
                    if( camera.getObjectDetector() != null ){
                        objectDetectionManager.detect( new Object[]{camera, System.nanoTime()} );
                    }
                }

                @Override
                public void onCaptureByFPS(CameraAdvanced camera) { }

                @Override
                public void onDemographic(List<String[]> demographic, Map<String, Integer> increasedCount, CameraAdvanced camera) {
                    listener.onDemographic(demographic, increasedCount, camera);
                }
            });
            cameras.get( camindex ).start();
        }catch( Exception ex){
            ex.printStackTrace();
        }
    }
    
    private synchronized void process( CameraAdvanced camera ){
        if( camera.getLastCapture() == null ) {
            listener.onFailed( camera, camera.getArrayIndex());
            return;
        }
        
        listener.onCapture( camera, camera.getLastCapture(), camera.getArrayIndex() );
        java.util.List<opencv_core.Mat> result = new java.util.ArrayList<>();
        

        listener.onDetectedFaces( result , camera, camera.getArrayIndex() );
        BufferedImage bimg = new Java2DFrameConverter( ).convert( converter.convert(camera.getLastCapture()) );
        listener.onCapture( camera, bimg, camera.getArrayIndex() );
    }
    
    public void restart(java.util.List<CameraAdvanced> cameras){
        stop();
        System.gc();
        this.cameras = cameras;
        start();
    }
    public void stop(){
        for( int i = 0; i < cameras.size(); i++ ){
            stop(i);
        }
    }
    public void stop( int index ){
        cameras.get( index ).close();
    }
        
    public void setListener( FaceListener listener) {
        this.listener = listener;
    } 
    public static void main( String args[]){
        new CameraManager( null );
    }
}
