/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.client;


import com.ml.vision.autoface.util.Constants;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.util.List;
import static org.bytedeco.javacpp.opencv_core.CV_32F;
import static org.bytedeco.javacpp.opencv_imgcodecs.imwrite;

/**
 *
 * @author apple
 */
public class FeatureExtractor {
        
    private String host;
    private int port;
    private java.util.List<opencv_core.Mat> featureListMat;
    private java.util.List<float[]> featureListFloat;
    private java.util.List<String[]> expressionList;
    
    public FeatureExtractor( String host, int port ){
        this.host = host;
        this.port = port;
    }

    public List<opencv_core.Mat> getFeatureListMat() {
        return featureListMat;
    }

    public List<float[]> getFeatureListFloat() {
        return featureListFloat;
    }

    public List<String[]> getExpressionList() {
        return expressionList;
    }

    public void extract(java.util.ArrayList<opencv_core.Mat> faceList, java.io.File parent, boolean expression ){

        if( featureListMat != null ){
            featureListMat.clear();
            featureListFloat.clear();
            expressionList.clear();
        }else{
            featureListMat = new java.util.ArrayList<>();
            featureListFloat = new java.util.ArrayList<>();
            expressionList = new java.util.ArrayList<>();
        }

        Socket clientSocket = null;
        try{
            clientSocket = new Socket( host, port );
            clientSocket.setReuseAddress(true);

            clientSocket.setReceiveBufferSize( Constants.BUFFER_SIZE );
            clientSocket.setSendBufferSize( Constants.BUFFER_SIZE );

            clientSocket.setSoTimeout( Constants.SO_TIMEOUT );
            DataInputStream dis = new DataInputStream(new BufferedInputStream( clientSocket.getInputStream()));
            DataOutputStream dos = new DataOutputStream(new BufferedOutputStream( clientSocket.getOutputStream()));

            dos.writeUTF("extract");// keyword
            dos.flush();
            dos.writeUTF( "," );// extra data
            dos.flush( );
            dos.writeInt(faceList.size());// total files
            dos.flush();

            int netSize = dis.readInt();// network vector size
            
            for( opencv_core.Mat image : faceList ){

                File f = new File( parent, "tmp-p.jpg" );
                if( f.exists() )
                    f.delete();
                f.createNewFile( );
                imwrite( f.getAbsolutePath(), image );
                byte features[] = send( dos, f, dis, expression );
                float f2[] = decode ( features );
                featureListFloat.add(f2);
                featureListMat.add( floatToMat( f2, netSize ) );
            }

            dos.close();
            dis.close();

        }catch (Exception ex){ex.printStackTrace();}
        finally{
            try {
                if( clientSocket!= null )
                    clientSocket.close();
            } catch ( Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private byte[] send( DataOutputStream dos, File img, DataInputStream dis, boolean expression ) throws Exception{

        byte features[] = null;
        dos.writeLong( img.length() );
        dos.flush();

        InputStream ios = null;
        try {
            byte[] buffer = new byte[Constants.BUFFER_SIZE];
            ios = new FileInputStream( img );
            int read = 0;
            while ((read = ios.read(buffer)) != -1) {
                dos.write(buffer, 0, read);
                dos.flush( );
            }
            ios.close();

            long remaningByte = dis.readLong( );
            System.out.println("Byte length=" + remaningByte);
            ByteArrayOutputStream ous = null;
            byte[] buff = new byte[Constants.BUFFER_SIZE];
            ous = new ByteArrayOutputStream( );
            read = 0;
            while ( remaningByte>0 &&
                    ( read = dis.read(buff,0, (int)Math.min( buff.length, remaningByte )) ) != -1){
                ous.write( buff, 0, read );
                remaningByte -= read;
            }
            ous.close();
            features = ous.toByteArray();

            dos.writeBoolean(expression);
            dos.flush();
            if( expression ){
                String strExpression = dis.readUTF();
                expressionList.add( strExpression.split(",") );
            }

        }catch (Exception ex){ex.printStackTrace();}

        return features;
    }

    public static byte[] encode (float floatArray[]) {
        byte byteArray[] = new byte[floatArray.length*4];

        // wrap the byte array to the byte buffer
        ByteBuffer byteBuf = ByteBuffer.wrap(byteArray);

        // create a view of the byte buffer as a float buffer
        FloatBuffer floatBuf = byteBuf.asFloatBuffer();

        // now put the float array to the float buffer,
        // it is actually stored to the byte array
        floatBuf.put (floatArray);

        return byteArray;
    }

    public static float[] decode (byte byteArray[]) {
        float floatArray[] = new float[byteArray.length/4];

        // wrap the source byte array to the byte buffer
        ByteBuffer byteBuf = ByteBuffer.wrap(byteArray);

        // create a view of the byte buffer as a float buffer
        FloatBuffer floatBuf = byteBuf.asFloatBuffer();

        // now get the data from the float buffer to the float array,
        // it is actually retrieved from the byte array
        floatBuf.get (floatArray);

        return floatArray;
    }

    public static float[] matToFloat( opencv_core.Mat feature, int size ){

        FloatIndexer indexer  = feature.createIndexer();
        float fArr[] = new float[ size ];
        indexer.get(0, fArr);

        return fArr;
    }

    public static opencv_core.Mat floatToMat( float f2[], int size ){
        opencv_core.Mat dest = new opencv_core.Mat( new opencv_core.Size( size, 1), CV_32F );
        FloatIndexer indexer2  = dest.createIndexer();
        indexer2.put(0, f2);

        return dest;
    }
}
