/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.client;


import com.ml.vision.autoface.util.Constants;
import java.awt.image.BufferedImage;
import org.bytedeco.javacpp.opencv_core;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.Socket;
import java.util.List;
import javax.imageio.ImageIO;
import static org.bytedeco.javacpp.opencv_imgcodecs.imwrite;

/**
 *
 * @author apple
 */
public class MultipleRecognizer {
        
    private String host;
    private int port;
    private java.util.List<String[]> resultList;
    private java.util.List<BufferedImage> photoList;
    private java.util.List<String[]> expressionList;
    
    public MultipleRecognizer( String host, int port ){
        this.host = host;
        this.port = port;
    }

    public List<String[]> getResultList() {
        return resultList;
    }

    public List<BufferedImage> getPhotoList() {
        return photoList;
    }
    
    public List<String[]> getExpressionList() {
        return expressionList;
    }

    public void extract(java.util.ArrayList<opencv_core.Mat> faceList, java.io.File parent, boolean expression ){

        if( resultList != null ){
            resultList.clear();
            photoList.clear();
            expressionList.clear();
        }else{
            resultList = new java.util.ArrayList<>();
            photoList = new java.util.ArrayList<>();
            expressionList = new java.util.ArrayList<>();
        }

        Socket clientSocket = null;
        try{
            clientSocket = new Socket( host, port );
            clientSocket.setReuseAddress(true);

            clientSocket.setReceiveBufferSize( Constants.BUFFER_SIZE );
            clientSocket.setSendBufferSize( Constants.BUFFER_SIZE );

            clientSocket.setSoTimeout( Constants.SO_TIMEOUT );
            DataInputStream dis = new DataInputStream(new BufferedInputStream( clientSocket.getInputStream()));
            DataOutputStream dos = new DataOutputStream(new BufferedOutputStream( clientSocket.getOutputStream()));

            dos.writeUTF("multirecognize");// keyword
            dos.flush();
            dos.writeUTF( "," );// extra data
            dos.flush( );
            dos.writeInt(faceList.size());// total files
            dos.flush();

            for( opencv_core.Mat image : faceList ){

                File f = new File( parent, "tmp-p.jpg" );
                if( f.exists() )
                    f.delete();
                f.createNewFile( );
                imwrite( f.getAbsolutePath(), image );
                send( dos, f, dis, expression );
            }

            dos.close();
            dis.close();

        }catch (Exception ex){ex.printStackTrace();}
        finally{
            try {
                if( clientSocket!= null )
                    clientSocket.close();
            } catch ( Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private void send( DataOutputStream dos, File img, DataInputStream dis, boolean expression ) throws Exception{

        dos.writeLong( img.length() );
        dos.flush();

        InputStream ios = null;
        try {
            // sending image file
            byte[] buffer = new byte[Constants.BUFFER_SIZE];
            ios = new FileInputStream( img );
            int read = 0;
            while ((read = ios.read(buffer)) != -1) {
                dos.write(buffer, 0, read);
                dos.flush( );
            }
            ios.close();
            // end of sending image file
            
            // reading recognition result
            String resultData = dis.readUTF();
            resultList.add(resultData.split(","));
            // end of reading recongnition result
            
            // checking profile image
            if(dis.readBoolean()){
            
                // reading profile image
                long remaningByte = dis.readLong( );
                System.out.println("Byte length=" + remaningByte);
                ByteArrayOutputStream ous = null;
                byte[] buff = new byte[Constants.BUFFER_SIZE];
                ous = new ByteArrayOutputStream( );
                read = 0;
                while ( remaningByte>0 &&
                        ( read = dis.read(buff,0, (int)Math.min( buff.length, remaningByte )) ) != -1){
                    ous.write( buff, 0, read );
                    remaningByte -= read;
                }
                ous.close();
                BufferedImage profileImages = ImageIO.read( new ByteArrayInputStream( ous.toByteArray()) );
                photoList.add(profileImages);
                // end of reading profile image
           
                
            }else{
                photoList.add( null );
            }
            
            // reading expression
            dos.writeBoolean(expression);
            dos.flush();
            if( expression ){
                String strExpression = dis.readUTF();
                expressionList.add( strExpression.split(",") );
            }
            // end of reading expression

        }catch (Exception ex){ex.printStackTrace();}

    }

}
