/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.client;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imwrite;

/**
 *
 * @author apple
 */
public class Recognition extends Thread implements Runnable{
        
    private Socket clientSocket;
    private opencv_core.Mat face;
    private String host;
    private int port;
    private RecognitionListener listener;
    
    
    public interface RecognitionListener{
        public void failed( String status );
        public void success( String id, String name, float distance, float accept, boolean match );
    }
    
    @Override
        public void run(){
            
            try {
                clientSocket = new Socket( host , port);
                clientSocket.setReuseAddress(true);
                DataInputStream dis = new DataInputStream(new BufferedInputStream( clientSocket.getInputStream()));
                DataOutputStream dos = new DataOutputStream(new BufferedOutputStream( clientSocket.getOutputStream()));

                dos.writeUTF("recognize");
                dos.flush();
                dos.writeUTF("1");
                dos.flush();
                
                try{
                    java.io.File f1 = new java.io.File("rtmp.jpg");
                    f1.createNewFile();
                    imwrite( f1.getAbsolutePath(), face.clone() );
                    
                    dos.writeLong( f1.length() );
                    dos.flush();
                    
                    byte[] buffer = new byte[4096];
                    InputStream ios = new FileInputStream( f1 );
                    int totalByte = 0;
                    int read = 0;
                    while ((read = ios.read(buffer)) != -1) {
                        dos.write(buffer, 0, read);
                        dos.flush( );
                        totalByte += read;
                    }
                    ios.close();
                    
                    System.out.println(f1.length() +"," + totalByte);
                }catch(Exception ex){ex.printStackTrace();}
                
                String response = dis.readUTF( );
                System.out.println( "Response:" + response );
                dos.close();
                dis.close();
                
                if( response.equalsIgnoreCase("failed") )
                    listener.failed(response);
                else{
                    String data[] = response.split(",");
                    String id = data[0];
                    String name = data[1];
                    float distance = Float.parseFloat(data[2]);
                    float accept = Float.parseFloat(data[3]);
                    boolean match = distance < accept;
                    listener.success( id, name, distance, accept, match );
                }
                
            } catch (Exception ex) {
                listener.failed(ex.getLocalizedMessage());
                Logger.getLogger(Recognition.class.getName()).log(Level.SEVERE, null, ex);
            }finally{
                try {
                    clientSocket.close();
                } catch (IOException ex) {
                listener.failed(ex.getLocalizedMessage());
                    Logger.getLogger(Recognition.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    
    public void setListener( RecognitionListener listener ){
        this.listener = listener;
    }
}
