/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.client.remote;

import com.ml.vision.autoface.util.Constants;
import java.io.DataOutputStream;
import java.io.File;
import java.net.HttpURLConnection;
import java.net.URL;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imwrite;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author apple
 */
public class Recognizer {

    private static final String URL_NAME = "/recognize";
    private File parent;
    private RecognizerListener listener;
    private boolean lock = false;
    private String urlAddress="", param="";
    private String demographic = "false";

    public interface RecognizerListener{
        public void failed();
        public void success( java.util.List<ResultObject> result );
        public void lock( boolean lock );
    }

    public Recognizer( ){

        parent = new File( Constants.TMP_DIR);
        if (!parent.exists())
            parent.mkdir();
    }

    public void recognize( String url, String param, java.util.List< opencv_core.Mat> faceList ){

        this.urlAddress = url;
        this.param = param;
        if( lock ){
            listener.lock( true );
            listener.failed();
            return;
        }

        listener.lock( true );
        lock = true;

        try {
            
            JSONParser parser = new JSONParser();
            JSONObject objRequest = (JSONObject) parser.parse(param);
            JSONArray images = new JSONArray(  );
            demographic = objRequest.get("demograph").toString();

            int count = 0;
            for( opencv_core.Mat image : faceList ){
                File f = new File( parent, count + "-tmp-p.jpg" );
                if( f.exists() )
                    f.delete();
                f.createNewFile( );
                
                imwrite( f.getAbsolutePath(), image );
                count++;
                image.release();

                String base64ImageString = Base64Image.encoder( f.getAbsolutePath() );
                JSONObject objimage = new JSONObject( );
                objimage.put("image", base64ImageString);
                images.add(objimage);

                File fDecode = new File( parent, count + "-tmp-decode.jpg" );
                Base64Image.decoder(base64ImageString, fDecode.getAbsolutePath() );
            }

            objRequest.put("images", images);
            this.param = objRequest.toString();

            sendPost();

        } catch (Exception e) {
            listener.failed();
            listener.lock( false );
            lock = false;
            e.printStackTrace();
        }

//        ConnectionRecognizer connectionRecognizer = new ConnectionRecognizer( url, param, faceList, parent );
//        String result = connectionRecognizer.recognize();
    }


    private void sendPost( ) {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    URL url = new URL( urlAddress + URL_NAME );
                    HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                    conn.setRequestMethod("POST");
//                    conn.setRequestProperty("Content-Type", "text/html;charset=UTF-8");
                    conn.setRequestProperty("Content-Type", "application/json;charset=UTF-8");
                    conn.setRequestProperty("Accept","application/json");
//                    conn.setRequestProperty("Accept","text/html");
                    conn.setDoOutput(true);
                    conn.setDoInput(true);


                    DataOutputStream os = new DataOutputStream(conn.getOutputStream());
                    os.writeBytes( param );
                    os.flush();
                    os.close();

                    System.out.println(String.valueOf(conn.getResponseCode()));
                    System.out.println( conn.getResponseMessage() );

                    StringBuilder sb = new StringBuilder();
                    java.io.DataInputStream din = new java.io.DataInputStream( conn.getInputStream() );
                    java.io.InputStreamReader reader = new java.io.InputStreamReader( din, "UTF-8" );
                    java.io.BufferedReader bf = new java.io.BufferedReader(reader);
                    String line = "";
                    while ( (line = bf.readLine()) != null ){
                        if( line == null ) break;
                        sb.append( line );
                    }
                    din.close( );

                    conn.disconnect();
                    response( sb.toString() );
                } catch (Exception e) {
                    listener.failed();
                    listener.lock( false );
                    lock = false;
                    e.printStackTrace();
                }
            }
        });

        thread.start();
    }

    private void response( String result){
        java.util.List<ResultObject> lstResult = new java.util.ArrayList<>();

        try {
            if( result.isEmpty()){
                listener.failed();
                listener.lock( false );
                lock = false;
            }else {
                System.out.println(result);
                JSONParser parser = new JSONParser();
                JSONArray ja = ( JSONArray ) parser.parse(result);
                
                for (int i = 0; i < ja.size(); i++) {

                    JSONObject objResult = (JSONObject) ja.get(i);

                    if( objResult.get("result").toString().equalsIgnoreCase("true")) {
                        ResultObject resultObject = new ResultObject(Boolean.parseBoolean(objResult.get("result").toString()),
                                Integer.parseInt(objResult.get("id").toString()), objResult.get("name").toString(),
                                Long.parseLong(objResult.get("date").toString()), Integer.parseInt(objResult.get("linkid").toString()),
                                Double.parseDouble(objResult.get("distance").toString()), Float.parseFloat(objResult.get("accept").toString()),
                                objResult.get("profile").toString() );

                        if( demographic.equalsIgnoreCase("true")){
                            resultObject.setAge( objResult.get("age").toString() );
                            resultObject.setGender( objResult.get("gender").toString() );
                            resultObject.setEmotion( objResult.get("emotion").toString() );
                        }
                        lstResult.add(resultObject);

                    }else{
                        ResultObject resultObject = new ResultObject(Boolean.parseBoolean(objResult.get("result").toString()),
                                objResult.get("name").toString(), Float.parseFloat(objResult.get("accept").toString()) );
                        lstResult.add(resultObject);
                    }
                }

                listener.success( lstResult );
                listener.lock( false );
                lock = false;
            }
        } catch (Exception e) {
            listener.failed();
            listener.lock( false );
            lock = false;
            e.printStackTrace();
        }
    }

    public void setListener( RecognizerListener listener ){
        this.listener = listener;
    }

    public class ResultObject{
        private boolean result;
        private int id;
        private String name;
        private long date;
        private int linkid;
        private double distance;
        private float accept;
        private String profile;
        private String age;
        private String gender;
        private String emotion;

        public ResultObject(boolean result, String name, float accept) {
            this.result = result;
            this.name = name;
            this.accept = accept;
        }

        public ResultObject(boolean result, int id, String name, long date, int linkid, double distance, float accept, String profile) {
            this.result = result;
            this.id = id;
            this.name = name;
            this.date = date;
            this.linkid = linkid;
            this.distance = distance;
            this.accept = accept;
            this.profile = profile;
        }

        public ResultObject(boolean result, int id, String name, long date, int linkid, double distance, float accept, String profile, String age, String gender, String emotion) {
            this.result = result;
            this.id = id;
            this.name = name;
            this.date = date;
            this.linkid = linkid;
            this.distance = distance;
            this.accept = accept;
            this.profile = profile;
            this.age = age;
            this.gender = gender;
            this.emotion = emotion;
        }

        public boolean isResult() {
            return result;
        }

        public void setResult(boolean result) {
            this.result = result;
        }

        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public long getDate() {
            return date;
        }

        public void setDate(long date) {
            this.date = date;
        }

        public int getLinkid() {
            return linkid;
        }

        public void setLinkid(int linkid) {
            this.linkid = linkid;
        }

        public double getDistance() {
            return distance;
        }

        public void setDistance(double distance) {
            this.distance = distance;
        }

        public float getAccept() {
            return accept;
        }

        public void setAccept(float accept) {
            this.accept = accept;
        }

        public String getProfile() {
            return profile;
        }

        public void setProfile(String profile) {
            this.profile = profile;
        }

        public String getAge() {
            return age;
        }

        public void setAge(String age) {
            this.age = age;
        }

        public String getGender() {
            return gender;
        }

        public void setGender(String gender) {
            this.gender = gender;
        }

        public String getEmotion() {
            return emotion;
        }

        public void setEmotion(String emotion) {
            this.emotion = emotion;
        }
    }

}
