/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.client.remote;

import com.ml.vision.autoface.util.Constants;
import java.io.DataOutputStream;
import java.io.File;
import java.net.HttpURLConnection;
import java.net.URL;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imwrite;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author apple
 */
public class Register {

    private static final String URL_NAME = "/register";
    private File parent;
    private RegisterListener listener;
    private boolean lock = false;
    private String urlAddress="", param="";

    public interface RegisterListener{
        public void failed();
        public void success( JSONObject result );
        public void lock( boolean lock );
    }

    public Register( ){

        parent = new File( Constants.TMP_DIR);
        if (!parent.exists())
            parent.mkdir();
    }

    public void register( String url, String param, opencv_core.Mat face, opencv_core.Mat profile ){

        this.urlAddress = url;
        this.param = param;
        if( lock ){
            listener.lock( true );
            listener.failed();
            return;
        }

        listener.lock( true );
        lock = true;

        try {
            
            JSONParser parser = new JSONParser();
            JSONObject objRequest = (JSONObject) parser.parse(param);
            
            File faceFile = new File( parent, System.nanoTime() + "-face.jpg" );
            if(faceFile.exists()) faceFile.delete();
            faceFile.createNewFile( );
            imwrite( faceFile.getAbsolutePath(), face );
            String base64FaceImageString = Base64Image.encoder( faceFile.getAbsolutePath() );
            objRequest.put("face", base64FaceImageString );
            
            File profileFile = new File( parent, System.nanoTime() + "-profile.jpg" );
            if(profileFile.exists()) profileFile.delete();
            profileFile.createNewFile( );
            imwrite( profileFile.getAbsolutePath(), profile );
            String base64ProfileImageString = Base64Image.encoder( profileFile.getAbsolutePath() );
            objRequest.put("profile", base64ProfileImageString );
            this.param = objRequest.toString();

            face.release();
            profile.release();
            sendPost();

        } catch (Exception e) {
            listener.failed();
            listener.lock( false );
            lock = false;
            e.printStackTrace();
        }
    }


    private void sendPost( ) {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    URL url = new URL( urlAddress + URL_NAME );
                    HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                    conn.setRequestMethod("POST");
//                    conn.setRequestProperty("Content-Type", "text/html;charset=UTF-8");
                    conn.setRequestProperty("Content-Type", "application/json;charset=UTF-8");
                    conn.setRequestProperty("Accept","application/json");
//                    conn.setRequestProperty("Accept","text/html");
                    conn.setDoOutput(true);
                    conn.setDoInput(true);


                    DataOutputStream os = new DataOutputStream(conn.getOutputStream());
                    os.writeBytes( param );
                    os.flush();
                    os.close();

                    System.out.println(String.valueOf(conn.getResponseCode()));
                    System.out.println( conn.getResponseMessage() );

                    StringBuilder sb = new StringBuilder();
                    java.io.DataInputStream din = new java.io.DataInputStream( conn.getInputStream() );
                    java.io.InputStreamReader reader = new java.io.InputStreamReader( din, "UTF-8" );
                    java.io.BufferedReader bf = new java.io.BufferedReader(reader);
                    String line = "";
                    while ( (line = bf.readLine()) != null ){
                        if( line == null ) break;
                        sb.append( line );
                    }
                    din.close( );
                    conn.disconnect();
                    
                    response( sb.toString() );
                } catch (Exception e) {
                    listener.failed();
                    listener.lock( false );
                    lock = false;
                    e.printStackTrace();
                }
            }
        });

        thread.start();
    }

    private void response( String result ){

        System.out.println( result );
        
        try {
            if( result.isEmpty()){
                listener.failed();
                listener.lock( false );
                lock = false;
            }else {
                JSONParser parser = new JSONParser();
                JSONObject responseJson = ( JSONObject ) parser.parse( result );
                listener.success( responseJson );
                listener.lock( false );
                lock = false;
            }
        } catch (Exception e) {
            listener.failed();
            listener.lock( false );
            lock = false;
            e.printStackTrace();
        }
    }

    public void setListener( RegisterListener listener ){
        this.listener = listener;
    }
}
