/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.database;


import com.ml.vision.autoface.util.Constants;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.CV_32F;
import static org.bytedeco.javacpp.opencv_core.norm;
import org.sqlite.SQLiteConfig;

/**
 *
 * @author apple
 */
public class SQLiteManager {
    public static final String DATABASE_DIR = "data/";
    public static final String DATABASE_URL =DATABASE_DIR + "autoface.db3";
    
    /*
    CREATE TABLE [foto](
  [_id] INTEGER PRIMARY KEY ASC ON CONFLICT ROLLBACK NOT NULL UNIQUE, 
  [label] TEXT NOT NULL, 
  [rdate] DATE, 
  [profile] BLOB, 
  [fs] BLOB NOT NULL, 
  [fu] BLOB, 
  [fd] BLOB, 
  [fl] BLOB, 
  [fr] BLOB, 
  [ffs128] BLOB, 
  [ffu128] BLOB, 
  [ffd128] BLOB, 
  [ffl128] BLOB, 
  [ffr128] BLOB, 
  [ffs2622] BLOB, 
  [ffu2622] BLOB, 
  [ffd2622] BLOB, 
  [ffl2622] BLOB, 
  [ffr2622] BLOB, 
  [sid] INTEGER);

CREATE INDEX [idx_label] ON [foto]([label]);
CREATE UNIQUE INDEX [idx_id] ON [foto]([_id]);
  */
    public java.sql.Connection getCommection(){
        
        java.sql.Connection conn=null;
        try {
            Class.forName("org.sqlite.JDBC");
            SQLiteConfig config = new SQLiteConfig();
            config.setBusyTimeout( 1000 * 15 );
            config.setEncoding( SQLiteConfig.Encoding.UTF_8 );
            config.setJournalMode( SQLiteConfig.JournalMode.DELETE );
            config.setCacheSize(-2000);
            config.setJounalSizeLimit(-1);
            config.setMaxPageCount(1073741823);
            config.setPageSize(4096);
            config.enforceForeignKeys(true);
            config.setLockingMode(SQLiteConfig.LockingMode.NORMAL);
            config.setSynchronous(SQLiteConfig.SynchronousMode.FULL);
            
            conn = java.sql.DriverManager.getConnection( "jdbc:sqlite:" + DATABASE_URL, config.toProperties() );
        } catch ( Exception ex) {
            Logger.getLogger( SQLiteManager.class.getName()).log(Level.SEVERE, null, ex);
        }
        return conn;
    }
        
    //_id, label, rdate, profile, fs, ffs2622, sid    
    public Object[] searchFaceV2( opencv_core.Mat inputFeature, int limit, int offset ){
        
        Object resultObj[] = new Object[2];
        String resultData [] = {"-1","Unknown","10000"};
        resultObj[0] = resultData;
        
        try{
            String sql = "select _id, label, rdate, profile, ffs2622, sid from [foto] where [_status]=1";
            if( limit > 0 )
                sql +=" limit " + limit +" offset " + offset;
            java.sql.PreparedStatement statement = getCommection().prepareStatement( sql );
            java.sql.ResultSet rs = statement.executeQuery();
            
            while( rs.next() ){
                
                int _id = rs.getInt("_id");
                String name = rs.getString("label");
                long rDate = rs.getDate("rdate").getTime();
                byte profile[] =rs.getBytes("profile");
                byte photo[] = rs.getBytes("ffs2622");
                int linkId = rs.getInt("sid");
                
                //BufferedImage profileImages = ImageIO.read( new ByteArrayInputStream( profile) );
                
                float f2[] = decode ( photo );
                opencv_core.Mat dest = new opencv_core.Mat( new opencv_core.Size( inputFeature.cols(), 1), CV_32F );
                FloatIndexer indexer2  = dest.createIndexer();
                indexer2.put(0, f2);
                            
                double distance = norm( inputFeature,  dest ) / 100;
                //System.out.println("Name="+name +", distance=" + distance);
                if( distance < Constants.VGG_ACCEPTANCE_LEVEL ){
                    resultData = new String[]{ _id +"", name, rDate+"", linkId+"", distance +"" };
                    resultObj[0] = resultData;
                    resultObj[1] = profile;
                    break;
                }
                
                dest.release();
                dest.close();
            }
            
            rs.close();
            close();
            inputFeature.release( );
            inputFeature.close( );
            
        }catch(Exception ex){ ex.printStackTrace(); }
        
        return resultObj;
    }  
      
    public float[] decode (byte byteArray[]) { 
        float floatArray[] = new float[byteArray.length/4]; 

        // wrap the source byte array to the byte buffer 
        ByteBuffer byteBuf = ByteBuffer.wrap(byteArray); 

        // create a view of the byte buffer as a float buffer 
        FloatBuffer floatBuf = byteBuf.asFloatBuffer(); 

        // now get the data from the float buffer to the float array, 
        // it is actually retrieved from the byte array 
        floatBuf.get (floatArray); 

        return floatArray; 
    } 
     
    public int getPhotoCount( String customSQL ){
        int total = 0;
        
        try{
            String sql = "select count(*) from [foto]";
            
            if( customSQL != null && !customSQL.isEmpty())
                sql = customSQL;
            
            java.sql.PreparedStatement statement = getCommection().prepareStatement(sql);
            java.sql.ResultSet rs = statement.executeQuery();
            while( rs.next()){
                total = rs.getInt(1);
            }
            rs.close();
            close( );
            
        }catch(Exception ex){ }
        
        return total;
    }
     
    public int getPhotoMaxId( String customSQL ){
        int total = 0;
        
        try{
            String sql = "select max(_id) from [foto]";
            
            if( customSQL != null && !customSQL.isEmpty())
                sql = customSQL;
            
            java.sql.PreparedStatement statement = getCommection().prepareStatement(sql);
            java.sql.ResultSet rs = statement.executeQuery();
            while( rs.next()){
                total = rs.getInt(1);
            }
            rs.close();
            close( );
            
        }catch(Exception ex){ }
        
        return total;
    }
        
    /**
     * This is database version 2 saving code
     * @param name
     * @param feature 2622 features vector
     * @param profilePath path to profile photo
     * @param photoPath path to registered face photo
     * @param linkId the ID from other programs if exist
     * @return number of effected rows
     */
    public int savePhotoV2( String name, byte feature[], String profilePath, String photoPath, int linkId ){
        int id = getPhotoMaxId( null ) + 1;
        int result = savePhotoV2( id, name, feature, profilePath, photoPath, linkId );
        if( result > 0 )
            return id;
        else
            return result;
    }
    
    /**
     * This is database version 2 saving code
     * @param id
     * @param name
     * @param feature 2622 features vector
     * @param profilePath path to profile photo
     * @param photoPath path to registered face photo
     * @param linkId the ID from other programs if exist
     * @return number of effected rows
     */
    public int savePhotoV2( int id, String name, byte feature[], String profilePath, String photoPath, int linkId ){
        int result = 0;
        
        try{
            String sql = "insert into [foto](_id, label, rdate, profile, fs, ffs2622, sid) "
                        + "values(?, ?, ?, ?, ?, ?, ?)";

            java.sql.PreparedStatement psmnt = getCommection().prepareStatement ( sql );
            
            psmnt.setInt( 1, id);// id
            psmnt.setString( 2,name );//name
            psmnt.setDate(3, new java.sql.Date(System.currentTimeMillis()));// register date
            psmnt.setBytes( 4, getImage(profilePath) ); // profile photo
            psmnt.setBytes( 5, getImage(photoPath) ); // registered face photo
            psmnt.setBytes( 6, feature ); // feature
            psmnt.setInt( 7, linkId );// link id from other software
            
            result = psmnt.executeUpdate( );
            psmnt.close( );
            close( );
            
        }catch(Exception ex){ex.printStackTrace();}
        
        return result;
    }
       
    /**
     * extracting data with SQL command, should close database connection "call close()" after processing
     * @param customSQL
     * @return ResultSet with data
     */
    public java.sql.ResultSet getData( String customSQL ){
        java.sql.ResultSet rs = null;
        
        try{
            java.sql.PreparedStatement statement = getCommection().prepareStatement( customSQL );
            rs = statement.executeQuery();
        }catch( Exception ex){}
        
        return rs;
    }
    
    public synchronized int execute( String sql){
        int result = -1;
        try{
                Statement stmt = getCommection().createStatement();
                result = stmt.executeUpdate(sql);
                stmt.close();
                close();
        }catch ( Exception ex){
            System.out.println( ex.toString() + " execute ( )");
        }
        
        return result;
    }
    
    public void close() throws SQLException{
        getCommection().close();
    }
    
    private byte[] getImage(String file){
        return getImage(new java.io.File(file));
    }
    
    private byte[] getImage(java.io.File file){
//        psmnt.setBytes( 9, getImage(image) ); // save the new image
//javax.swing.ImageIcon icon = new javax.swing.ImageIcon( dbimg );

        byte x[]=null;
        
        try{
            if( file == null ) return null;
            
            FileInputStream fis = new FileInputStream( file );
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] buf = new byte[1024];

            try{
                for ( int readNum; (readNum = fis.read(buf))  != -1;){
                    bos.write(buf, 0, readNum);
                }
                x = bos.toByteArray();
                bos.close();
            }catch ( Exception ex){

            }
            
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        
        return x;
    }

    public static void main( String args[] ){
        SQLiteManager sql = new SQLiteManager( );
        System.out.println(sql.getPhotoCount(null));
    }
}
