/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.detection;


import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bytedeco.javacpp.Loader;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.CV_32F;
import org.bytedeco.javacpp.opencv_core.Scalar;
import org.bytedeco.javacpp.opencv_core.Size;
import org.bytedeco.javacpp.opencv_dnn;
import static org.bytedeco.javacpp.opencv_dnn.blobFromImage;
import static org.bytedeco.javacpp.opencv_dnn.readNetFromCaffe;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;
import static org.bytedeco.javacpp.opencv_imgproc.CV_BGR2GRAY;
import static org.bytedeco.javacpp.opencv_imgproc.FONT_HERSHEY_PLAIN;
import static org.bytedeco.javacpp.opencv_imgproc.LINE_8;
import static org.bytedeco.javacpp.opencv_imgproc.cvtColor;
import static org.bytedeco.javacpp.opencv_imgproc.equalizeHist;
import static org.bytedeco.javacpp.opencv_imgproc.putText;
import static org.bytedeco.javacpp.opencv_imgproc.rectangle;
import static org.bytedeco.javacpp.opencv_imgproc.resize;
import org.bytedeco.javacpp.opencv_objdetect;


/**
 *
 * @author apple
 */
public class FaceDetection {
    
    public static final int ALGORITHM_CASCADE_CLASSIFIER = 0;
    public static final int ALGORITHM_CAFFE_MODEL = 1;
    public static final int IMG_SIZE = 250;
    public static final String PROTO_FILE = "model/deploy.prototxt";
    public static final String CAFFE_MODEL_FILE = "model/res10_300x300_ssd_iter_140000.caffemodel";
    public static final String CLASSIFIERNAME = "model/haarcascade_frontalface_alt2.xml";
    private opencv_dnn.Net net = null;
    private opencv_objdetect.CascadeClassifier classifier = null;
    public static final int IMG_MIN_SIZE = 72;//128
    private java.util.List<opencv_core.Rect> lstRect = new java.util.ArrayList<>();
    private boolean rectangle = true;
    
    /**
     * Google Caffe deep learning algorithms
     * @param image input opencv_core.Mat image
     * @param isGrayMat true to convert to gray scale otherwise false
     * @param isRectangle true to render with rectangle otherwise false
     * @return detected faces ArrayList<opencv_core.Mat>
     */
    public java.util.List<opencv_core.Mat> caffemodel( opencv_core.Mat image, boolean isGrayMat, boolean isRectangle ){
        return caffemodel( image, isGrayMat, isRectangle, false );
    }
    
    public java.util.List<opencv_core.Mat> caffemodel( opencv_core.Mat image, boolean isGrayMat, boolean isRectangle, boolean sorted ){
                
        java.util.List<opencv_core.Mat> capture = new java.util.ArrayList<>();
        if( image == null ) return capture;
        lstRect.clear();
        
        if( net == null )
            net = readNetFromCaffe(PROTO_FILE, CAFFE_MODEL_FILE);
            
        resize(image, image, new opencv_core.Size(300, 300));//resize the image to match the input size of the model
        //create a 4-dimensional blob from image with NCHW (Number of images in the batch -for training only-, Channel, Height, Width) dimensions order,
        //for more detailes read the official docs at https://docs.opencv.org/trunk/d6/d0f/group__dnn.html#gabd0e76da3c6ad15c08b01ef21ad55dd8
        opencv_core.Mat blob = blobFromImage(image, 1.0, new opencv_core.Size(300, 300), new opencv_core.Scalar(104.0, 177.0, 123.0, 0), false, false, CV_32F);

        net.setInput(blob);//set the input to network model
        opencv_core.Mat output = net.forward();//feed forward the input to the netwrok to get the output matrix

        opencv_core.Mat ne = new opencv_core.Mat(new opencv_core.Size(output.size(3), output.size(2)), CV_32F, output.ptr(0, 0));//extract a 2d matrix for 4d output matrix with form of (number of detections x 7)

        FloatIndexer srcIndexer = ne.createIndexer(); // create indexer to access elements of the matric
        opencv_core.Mat greyMat = new opencv_core.Mat(image.rows(), image.cols() );
        cvtColor( image, greyMat, CV_BGR2GRAY);
                    
        for (int i = 0; i < output.size(3); i++) {//iterate to extract elements
            float confidence = srcIndexer.get(i, 2);
            if (confidence > .6) {
                try {
                    float f1 = srcIndexer.get(i, 3);
                    float f2 = srcIndexer.get(i, 4);
                    float f3 = srcIndexer.get(i, 5);
                    float f4 = srcIndexer.get(i, 6);
                    float tx = f1 * 300;//top left point's x
                    float ty = f2 * 300;//top left point's y
                    float bx = f3 * 300;//bottom right point's x
                    float by = f4 * 300;//bottom right point's y
                    
                    opencv_core.Rect rectFace = new opencv_core.Rect(new opencv_core.Point((int) tx, (int) ty), new opencv_core.Point((int) bx, (int) by));
                    if( isRectangle){
                        rectangle(image, rectFace, new opencv_core.Scalar( 255, 0, 0, 0 ), 4, LINE_8, 0);
                        // And now put it into the image:
                    }//print blue rectangle

                    try{
                        opencv_core.Mat capturedFace = new opencv_core.Mat( image, rectFace );
                        resize( capturedFace, capturedFace, new opencv_core.Size(IMG_SIZE, IMG_SIZE));
                        capture.add( capturedFace );
                        lstRect.add( rectFace );
//                        File f = new File("out/test" + index+"-"+ System.nanoTime() + ".jpg");
//                        f.createNewFile();
//                        imwrite( f.getAbsolutePath(), capturedFace );
                    }catch( Exception ex){
//                        ex.printStackTrace();
                    }
                } catch (Exception ex) {
                    Logger.getLogger( FaceDetection.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        System.gc();
        return capture;
    }
    
    /**
     * OpenCV machine learning algorithms
     * @param image input opencv_core.Mat image
     * @param isGrayMat true to convert to gray scale otherwise false
     * @param isRectangle true to render with rectangle otherwise false
     * @return detected faces ArrayList<opencv_core.Mat>
     */
//    int filecount=0;
    public java.util.List<opencv_core.Mat> cascadeClassifier( opencv_core.Mat image, boolean isGrayMat, boolean isRectangle ){
        return cascadeClassifier( image, isGrayMat, isRectangle, false );
    }
    
    public java.util.List<opencv_core.Mat> cascadeClassifier( opencv_core.Mat image, boolean isGrayMat, boolean isRectangle, boolean sorted ){
        rectangle = isRectangle;
        java.util.List<opencv_core.Mat> capture = new java.util.ArrayList<>( );
        if( image == null ) return capture;
        
        lstRect.clear();
        try{
            if (classifier == null) {
                // Preload the opencv_objdetect module to work around a known bug.
                Loader.load(opencv_objdetect.class);
                // We can "cast" Pointer objects by instantiating a new object of the desired class.
                classifier = new opencv_objdetect.CascadeClassifier( CLASSIFIERNAME );
            }
            
            opencv_core.Mat grayImage = new opencv_core.Mat(image.rows(), image.cols());
            cvtColor( image, grayImage, CV_BGR2GRAY);
            equalizeHist( grayImage, grayImage );
            opencv_core.RectVector faces = new opencv_core.RectVector();
            classifier.detectMultiScale( grayImage , faces, 1.25f, 3, 1,
                    new Size( IMG_MIN_SIZE, IMG_MIN_SIZE ),
                    new Size( 4 * IMG_SIZE, 4 * IMG_SIZE ) );
            
            for (long i = 0; i < faces.size(); i++) {
                
                opencv_core.Rect r = faces.get(i);
                if( r.height() > IMG_MIN_SIZE && r.width() > IMG_MIN_SIZE ){
                    
                    try{
                        r.y( r.y() - 5 );
                        r.height( r.height() + 5 );
                        r.x( r.x() - 5 );
                        r.width( r.width() + 5 );
                        
//                        r.y(r.y() - 30);
//                        r.height(r.height() + 50);
//                        r.x(r.x() - 30);
//                        r.width(r.width() + 50);
                        opencv_core.Mat capturedFace = new opencv_core.Mat( image, r );
//                        resize( capturedFace, capturedFace, new opencv_core.Size( 330, 330 ));
                        //resize( capturedFace, capturedFace, new opencv_core.Size( IMG_SIZE, IMG_SIZE ));
                        capture.add( capturedFace );
                        opencv_core.Rect cRect = new opencv_core.Rect(new opencv_core.Point( r.x(), r.y()), 
                                new opencv_core.Size( r.width(), r.height() ) );
                        lstRect.add(cRect);
                    }catch( Exception ex){
    //                    ex.printStackTrace();
                    }
                    
                    if( isRectangle() ){
                        rectangle( image, r, new opencv_core.Scalar(255, 0, 0, 0), 4, LINE_8, 0 );
//                        putText(image, "Moe Thant Oo", new opencv_core.Point( r.x(), r.y()), FONT_HERSHEY_PLAIN, 4.0, new Scalar(0, 255, 0, 4.0));
                    }
                }
            }
            if( sorted ){
                
                Collections.sort( lstRect, ( opencv_core.Rect m1, opencv_core.Rect m2) -> 
                        ( (m2.width() * m2.height()) - ( m1.width() * m1.height() ) ));
                
                Collections.sort( capture, (opencv_core.Mat m1, opencv_core.Mat m2) -> 
                        ( (m2.rows() * m2.cols()) - ( m1.rows() * m1.cols() ) ));
//                Collections.sort( capture, new Comparator<opencv_core.Mat>() {
//                    @Override
//                    public int compare( opencv_core.Mat m1, opencv_core.Mat m2) {
//                        return ( (m2.rows() * m2.cols()) - ( m1.rows() * m1.cols() ) );
//                    }
//                });
                
            }
            
        }catch( Exception e){
//            e.printStackTrace();
        }
        
        System.gc();
        return capture;
    }
    
    public java.util.List<opencv_core.Mat> detectFace( opencv_core.Mat image, boolean isGrayMat, 
            boolean isRectangle, int algorithm, boolean sorted ){
        if( algorithm == ALGORITHM_CASCADE_CLASSIFIER)
            return cascadeClassifier( image, isGrayMat, isRectangle, sorted );
        else
            return caffemodel( image, isGrayMat, isRectangle, sorted );
    }
    
    public java.util.List<opencv_core.Mat> detectFace( opencv_core.Mat image, boolean isGrayMat, boolean isRectangle, int algorithm){
        return detectFace( image, isGrayMat, isRectangle, algorithm, false );
    }

    public List<opencv_core.Rect> getLstRect() {
        return lstRect;
    }

    public boolean isRectangle() {
        return rectangle;
    }

    public void setRectangle(boolean rectangle) {
        this.rectangle = rectangle;
    }
    
}
