/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.detection;

/**
 *
 * @author apple
 */

import java.awt.image.BufferedImage;
import org.bytedeco.javacpp.indexer.Indexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.*;
import org.bytedeco.javacpp.opencv_dnn;
import static org.bytedeco.javacpp.opencv_dnn.*;
import org.bytedeco.javacpp.opencv_imgcodecs;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;
import static org.bytedeco.javacpp.opencv_imgproc.resize;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.bytedeco.javacv.OpenCVFrameConverter;

/**
 * The class responsible for recognizing gender. This class use the concept of CNN (Convolution Neural Networks) to
 * identify the gender of a detected face.
 *
 * @author Imesha Sudasingha
 */
public class GenderDetection {
//    https://github.com/IMS94/javacv-cnn-example
    
    private opencv_dnn.Net net = null;
    
    public static final String PROTO_FILE = "model/deploy_gendernet.prototxt";
    public static final String CAFFE_MODEL_FILE = "model/gender_net.caffemodel";
    
    public static void main(String args[]){
        GenderDetection genderDetector = new GenderDetection();
        Mat testImage = imread("trained/26193166243590.jpg", opencv_imgcodecs.IMREAD_ANYCOLOR);
            Gender gender = genderDetector.caffemodel( testImage );
            System.out.println("Gender="+gender);
    }
    public GenderDetection() {
        
        try{
            net = readNetFromCaffe(PROTO_FILE, CAFFE_MODEL_FILE);
        }catch(Exception ex){}
    }
    public Gender caffemodel( BufferedImage image ){
        return caffemodel( new OpenCVFrameConverter.ToIplImage().convertToMat( new Java2DFrameConverter( ).convert( image )) );
    }
    public int caffemodel( opencv_core.Mat image, int noAction ){
        try{
            resize(image, image, new opencv_core.Size(256,256));//resize the image to match the input size of the model
    //        normalize(image, image, 0, Math.pow(2, 8), NORM_MINMAX, -1, null);
            //create a 4-dimensional blob from image with NCHW (Number of images in the batch -for training only-, Channel, Height, Width) dimensions order,
            //for more detailes read the official docs at https://docs.opencv.org/trunk/d6/d0f/group__dnn.html#gabd0e76da3c6ad15c08b01ef21ad55dd8
            opencv_core.Mat blob = blobFromImage( image);

            net.setInput(blob, "data", 1.0, null);//set the input to network model
            opencv_core.Mat output = net.forward("prob" );//feed forward the input to the netwrok to get the output matrix

            Indexer indexer  = output.createIndexer(); // create indexer to access elements of the matric
    //                    System.out.println("Output Size="+indexer.getDouble(0, 0)+","+indexer.getDouble(0, 1));
            if (indexer.getDouble(0, 0) > indexer.getDouble(0, 1)) {
                    return 0;
                } else {
                    return 1;
                }
        }catch(Exception ex){}
        return -1;
    }
   
    public Gender caffemodel( opencv_core.Mat image ){
        int index = caffemodel( image, 0 );
        if( index == 0 )
            return Gender.MALE;
        else if( index == 1)
            return Gender.FEMALE;
        else
            return Gender.NOT_RECOGNIZED;
    }
    
    public enum Gender {
        MALE,
        FEMALE,
        NOT_RECOGNIZED
    }
}
