/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.detection.objects;



import java.util.Map;
import org.bytedeco.javacpp.opencv_core;

/**
 *
 * @author apple
 */
public class CentroidTracker {
    
    private java.util.Map<Long,TrackableObject> objects = new java.util.HashMap<>();
    private int maxDisappeared = 50;
    private int maxDistance = 50;
    
    private java.util.Map<String,Integer> mapCount = new java.util.TreeMap<>();
    
    public CentroidTracker( int maxDisappeared, int maxDistance ){
        this.maxDisappeared = maxDisappeared;
        this.maxDistance = maxDistance;
    }
    
    public TrackableObject add( opencv_core.Rect rect, float centroid, String className ){
        long id = System.currentTimeMillis();
        TrackableObject trackableObject = new TrackableObject( id, centroid, rect, true );
        trackableObject.setClassName( className );
        objects.put( id, trackableObject );
        
        return trackableObject;
    }
    
    public void remove( long objectID ){
        objects.remove(objectID);
    }
    
    public void check( ){
        try{
            for( java.util.Map.Entry<Long,TrackableObject> obj :  objects.entrySet() ){
                int disappeared = obj.getValue( ).getDisappeared( ) + 1;
                obj.getValue( ).setDisappeared( disappeared );
                if( disappeared > maxDisappeared )
                    remove( obj.getKey() );
            }
        }catch( Exception ex ){}
    }
    
    public TrackableObject update( opencv_core.Rect newObj, String className ){
        TrackableObject obj = null;
        
        try{
            obj = isNewObject( newObj, className );
            if( obj == null ){
                
                obj = add( newObj, 0.0f, className );
                if( mapCount.get( className ) == null ){
                    mapCount.put( className, 1 );
                }else{
                    mapCount.replace( className, mapCount.get(className) + 1 );
                }
                
            }else{
                //System.out.println(className+" Old object=");
            }
        }catch( Exception ex ){ ex.printStackTrace(); }
        
        return obj;
    }
    
    public TrackableObject isNewObject( opencv_core.Rect obj, String className ) {
        
        TrackableObject result = null;
        
        for ( java.util.Map.Entry<Long,TrackableObject> previousObj :  objects.entrySet() ) {
            if( previousObj.getValue().getClassName().equals(className)){
                if (isNear(obj, previousObj.getValue().getRect(), 3.0d )) {// 2.0d
                    result = previousObj.getValue();
                    previousObj.getValue().setRect( obj );
                    previousObj.getValue().setDisappeared( 0 );
                    //System.out.println(className+" disapppear="+previousObj.getValue().getDisappeared());
                    previousObj.getValue().setNewObject( false );
                    break;
                }
            }
        }

        return result;
    }
    
    private boolean isNear( opencv_core.Rect obj, opencv_core.Rect previousObj, double ratio) {
        
        opencv_core.Point personCenter = new opencv_core.Point( obj.x() + obj.width() / 2, obj.y()
                        + obj.height() / 2);

        opencv_core.Point previousCenter = new opencv_core.Point(previousObj.x()
                        + previousObj.width() / 2, previousObj.y()
                        + previousObj.height() / 2);

        opencv_core.Point maxAllowed = new opencv_core.Point(
                (int) (ratio * ( obj.width() + previousObj.width() )) / 2, 
                (int) ( ratio  * ( obj.height() + previousObj.height() )) / 2);

        opencv_core.Point movement = new opencv_core.Point(previousCenter.x() - personCenter.x(),
                        previousCenter.y() - personCenter.y());

        if (movement.x() < maxAllowed.x() && movement.y() < maxAllowed.y() ) {
                return true;
        }

        return false;
    }

    public Map<String, Integer> getMapCount() {
        return mapCount;
    }
    
    
}
