/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.detection.objects;


import com.ml.vision.autoface.detection.AgeDetection;
import com.ml.vision.autoface.detection.EmotionDetection;
import com.ml.vision.autoface.detection.FaceDetection;
import com.ml.vision.autoface.detection.GenderDetection;
import org.bytedeco.javacpp.DoublePointer;
import org.bytedeco.javacpp.IntPointer;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.CV_32F;
import static org.bytedeco.javacpp.opencv_core.minMaxLoc;
import org.bytedeco.javacpp.opencv_dnn;
import static org.bytedeco.javacpp.opencv_dnn.blobFromImage;
import static org.bytedeco.javacpp.opencv_imgproc.getTextSize;
import static org.bytedeco.javacpp.opencv_imgproc.putText;
import static org.bytedeco.javacpp.opencv_imgproc.rectangle;
import static org.bytedeco.javacpp.opencv_imgproc.FONT_HERSHEY_DUPLEX;
import static org.bytedeco.javacpp.opencv_imgproc.LINE_8;
/**
 *
 * @author apple
 */
public class Yolov3DarkNet extends Yolov3{
    
    private final int IN_WIDTH = 416;
    private final int IN_HEIGHT = 416;
    private final float WH_RATIO = (float)IN_WIDTH / IN_HEIGHT;
    private final double IN_SCALE_FACTOR = 0.00392;
    private final double MEAN_VAL = 127.5;
    private final double THRESHOLD = 0.6f;
   
           
    public Yolov3DarkNet( opencv_dnn.Net net, java.util.List<String> lstClassName ){
        this.net = net;
        this.lstClassName = lstClassName;
        
    }
           
    public Yolov3DarkNet( opencv_dnn.Net net, java.util.List<String> lstClassName, boolean count ){
        this.net = net;
        this.lstClassName = lstClassName;
        this.count = count;
        
    }
        
    public Yolov3DarkNet( opencv_dnn.Net net, java.util.List<String> lstClassName, boolean count, boolean demographic,
            FaceDetection detection, GenderDetection genderDetector, AgeDetection ageDetector, EmotionDetection emotionDetector ){
        this.net = net;
        this.lstClassName = lstClassName;
        this.count = count;
        this.demographic = demographic;
        this.detection = detection;
        this.genderDetector = genderDetector;
        this.ageDetector = ageDetector;
        this.emotionDetector = emotionDetector;
        
    }
    
    @Override
    public void detect( opencv_core.Mat frame ){
               
        if( isCount() && tracker == null )
            tracker = new CentroidTracker( maxDisappeared, maxDistance );
        
        if( isDemographic() ){
            lstDemographic = new java.util.ArrayList<>();
        }
        
        if( lstDemographic != null )
            lstDemographic.clear( );
                
        try{
//            resize( frame, frame, new opencv_core.Size( IN_WIDTH, IN_HEIGHT ));
            opencv_core.Mat blob = blobFromImage( frame, IN_SCALE_FACTOR,
            new opencv_core.Size(IN_WIDTH, IN_HEIGHT),
            new opencv_core.Scalar(0, 0, 0, 0), true, false, CV_32F );
            net.setInput(blob);
            opencv_core.MatVector result = new opencv_core.MatVector();
            net.forward( result, getOutputsNames( ));
//            net.forward( result, net.getUnconnectedOutLayersNames() );

            for (int i = 0; i < result.size(); ++i){
                opencv_core.Mat level = result.get(i);
                
                for (int j = 0; j < level.rows(); ++j){
                    
                    opencv_core.Mat row = level.row(j);
                    opencv_core.Mat scores = row.colRange( 5, level.cols() );
                    
                    DoublePointer minVal= new DoublePointer( new double[1] );
                    DoublePointer maxVal= new DoublePointer( new double[1] );
                    opencv_core.Point min = new opencv_core.Point();
                    opencv_core.Point max = new opencv_core.Point();
                    
                    minMaxLoc( scores, minVal, maxVal, min, max, null);
                    float confidence = (float) maxVal.get(0);

                    if( confidence > THRESHOLD){
                        int classId = max.x();
                        if( classId > classNames.length - 1 || classId < 0 ) continue;
                        String label = classNames[ classId ];
                        
                        if( this.lstClassName.contains(label) ){
                        
                            FloatIndexer indexer  = row.createIndexer();

                            int centerX = (int)( indexer.getDouble(0, 0, 0) * frame.cols());
                            int centerY = (int)( indexer.getDouble(0, 0, 1) * frame.rows());

                            int width   = (int)( indexer.getDouble(0, 0, 2) * frame.cols());
                            int height  = (int)( indexer.getDouble(0, 0, 3) * frame.rows());
                            int left    = centerX - width  / 2;
                            int top     = centerY - height / 2;
                        
                            // Draw rectangle around detected object.
                            opencv_core.Rect lr = new opencv_core.Rect( new opencv_core.Point( left, top ), 
                                new opencv_core.Point( left + width , top + height  ));

                            int[] baseLine = new int[1];
                            opencv_core.Size labelSize = getTextSize( label, FONT_HERSHEY_DUPLEX, 0.5, 1, baseLine);
                            // Draw background for label.
                            opencv_core.Rect bg = new opencv_core.Rect( 
                                    new opencv_core.Point(left, top - labelSize.height()), 
                                new opencv_core.Point(left + labelSize.width(), top + baseLine[0]) );

                            if( isRectangleObject() ){
                                rectangle( frame, lr, new opencv_core.Scalar(0, 255, 0, 0), 4, LINE_8, 0);                        
                                rectangle(frame, bg, new opencv_core.Scalar(0, 0, 0,0), 4, LINE_8, 0 );
                                // Write class name and confidence.
                                putText(frame, label, new opencv_core.Point(left, top),
                                        FONT_HERSHEY_DUPLEX, 1.1, new opencv_core.Scalar(0, 0, 200, 0));
                            }
                            
                            //System.out.println( "Name=" + label  );
                            if( tracker != null ){
                                // id>0 means that new object otherwise existing object
                                TrackableObject trackedObject = tracker.update( lr, label );
                                if( !trackedObject.isDemographDone() ){
                                    if( isDemographic() && detection != null && label.equals( DARKNET_OBJECT_PERSON))
                                       if( demograph( frame, lr, detection ) ) {trackedObject.setDemographDone(true);} 
                                }
                            }
                        }
                    }

                }
                level.release();
                level.close();
            }
            result.clear();
            result.close();
                        
            if( tracker != null ){
                tracker.check();
                checkInreasement( );
            }
//            imwrite ("yolov3.jpg", frame );
            
        }catch(Exception ex){ex.printStackTrace();}
    }

    @Override
    public void checkInreasement( ){
        try{
            if( lastCount.isEmpty() ){
                getCount( ).entrySet().forEach((entry) -> {
                    lastCount.put( entry.getKey(), entry.getValue() );
                    increasedCount.put( entry.getKey(), entry.getValue() );
                });
            }else{
                increasedCount.clear();
                
                getCount( ).entrySet().forEach((entry) -> {
                    int tmpCount = lastCount.getOrDefault( entry.getKey(), 0);
                    int increase = entry.getValue() - tmpCount;
                    if( increase > 0)
                        increasedCount.put( entry.getKey(), increase );    
                    
                    lastCount.replace( entry.getKey(), entry.getValue() );
                    
                });
                
            }
        }catch(Exception ex){ex.printStackTrace();}
    }
    
    @Override
    public boolean demograph( opencv_core.Mat frame, opencv_core.Rect rect, FaceDetection detection ){
        boolean result = false;
        
        try{
            opencv_core.Mat capturedFace = new opencv_core.Mat( frame, rect );
            java.util.List<opencv_core.Mat> capture = detection.detectFace( capturedFace, false, false, FaceDetection.ALGORITHM_CASCADE_CLASSIFIER, true );
            if( capture != null && !capture.isEmpty()){
                opencv_core.Mat firstFace = capture.get(0);

                String age = ageDetector.caffemodel( firstFace );
                String gender = genderDetector.caffemodel( firstFace ).toString();
                String emo = emotionDetector.caffemodel( firstFace );
                lstDemographic.add( new String[] {age, gender, emo} );
                System.out.println("Age="+age+", Gender=" + gender +", Emotion=" + emo);
                result = true;
            }
        }catch(Exception ex){}
        
        return result;
    }
    
    
    // Get the names of the output layers
    private opencv_core.StringVector getOutputsNames( ) {
        
//        opencv_core.StringVector namest = new opencv_core.StringVector( );
//        namest = net.getUnconnectedOutLayersNames();
        //        System.out.println("Layer size="+namest.size());
//        for( int i =0; i < namest.size(); i++)
//            System.out.println(namest.get(i).getString());


        opencv_core.StringVector names = new opencv_core.StringVector( );
        if (names.empty()) {
            //Get the indices of the output layers, i.e. the layers with unconnected outputs
            IntPointer outLayers = net.getUnconnectedOutLayers( );

            //get the names of all the layers in the network
            opencv_core.StringVector layersNames = net.getLayerNames();

            // Get the names of the output layers in names
            for( int i=0; outLayers.get(i) > 0 && outLayers.get(i) < 416; i++ ){
//                System.out.print("label=" + outLayers.get(i));
//                System.out.println(", Layer ="+ layersNames.get( outLayers.get(i) - 1).getString() +", label=" + outLayers.get(i));
                names.push_back( layersNames.get( outLayers.get(i) - 1));
            }
        }
        return names;
    }

//    public static void main( String args[] ){
//        opencv_core.Mat imgbgr1 = imread( "sample/IMG_1944_810.jpg" );
//        java.util.List<String> lstClassName = new java.util.ArrayList<>();
//        
//        lstClassName.add( DARKNET_OBJECT_PERSON );
//        lstClassName.add( DARKNET_OBJECT_BICYCLE );
//        lstClassName.add( DARKNET_OBJECT_CAR );
//        lstClassName.add( DARKNET_OBJECT_MOTORCYCLE );
//        lstClassName.add( DARKNET_OBJECT_AIRPLANE );
//        lstClassName.add( DARKNET_OBJECT_BUS );
//        lstClassName.add( DARKNET_OBJECT_TRAIN );
//        lstClassName.add( DARKNET_OBJECT_TRUCK );
//        
//        Yolov3DarkNet y3 = new Yolov3DarkNet(null, lstClassName, true, true, null );
//        y3.detect( imgbgr1 );
//    }
    
    public static final String[] classNames = {"person","bicycle","car","motorcycle","airplane","bus","train","truck",
        "boat","traffic light","fire hydrant","stop sign","parking meter",
        "bench","bird","cat","dog","horse","sheep","cow","elephant","bear","zebra","giraffe",
        "backpack","umbrella","handbag","tie","suitcase","frisbee","skis","snowboard","sports ball",
        "kite","baseball bat","baseball glove","skateboard","surfboard","tennis racket",
        "bottle","wine glass","cup","fork","knife","spoon","bowl",
        "banana","apple","sandwich","orange","broccoli","carrot",
        "hot dog","pizza","donut","cake","chair","couch","potted plant","bed","dining table","toilet",
        "tv","laptop","mouse","remote","keyboard","cell phone",
        "microwave","oven","toaster","sink","refrigerator","book","clock","vase","scissors","teddy bear","hair drier","toothbrush"};
    
    public static final String DARKNET_OBJECT_PERSON = "person";
    public static final String DARKNET_OBJECT_BICYCLE = "bicycle";
    public static final String DARKNET_OBJECT_CAR = "car";
    public static final String DARKNET_OBJECT_MOTORCYCLE = "motorcycle";
    public static final String DARKNET_OBJECT_AIRPLANE = "airplane";
    public static final String DARKNET_OBJECT_BUS = "bus";
    public static final String DARKNET_OBJECT_TRAIN = "train";
    public static final String DARKNET_OBJECT_TRUCK = "truck";
    public static final String DARKNET_OBJECT_BOAT = "boat";
    public static final String DARKNET_OBJECT_TRAFFIC_LIGHT = "traffic light";
    public static final String DARKNET_OBJECT_FIRE_HYDRANT = "fire hydrant";
    public static final String DARKNET_OBJECT_STOP_SIGN = "stop sign";
    public static final String DARKNET_OBJECT_PARKING_METER = "parking meter";
    public static final String DARKNET_OBJECT_BENCH = "bench";
    public static final String DARKNET_OBJECT_BIRD = "bird";
    public static final String DARKNET_OBJECT_CAT = "cat";
    public static final String DARKNET_OBJECT_DOG = "dog";
    public static final String DARKNET_OBJECT_HORSE = "horse";
    public static final String DARKNET_OBJECT_SHEEP = "sheep";
    public static final String DARKNET_OBJECT_COW = "cow";
    public static final String DARKNET_OBJECT_ELEPHANT = "elephant";
    public static final String DARKNET_OBJECT_BEAR = "bear";
    public static final String DARKNET_OBJECT_ZEBRA = "zebra";
    public static final String DARKNET_OBJECT_GIRAFFE = "giraffe";
    public static final String DARKNET_OBJECT_BACKPACK = "backpack";
    public static final String DARKNET_OBJECT_UMBRELLA = "umbrella";
    public static final String DARKNET_OBJECT_HANDBAG = "handbag";
    public static final String DARKNET_OBJECT_TIE = "tie";
    public static final String DARKNET_OBJECT_SUITCASE = "suitcase";
    public static final String DARKNET_OBJECT_FRISBEE = "frisbee";
    public static final String DARKNET_OBJECT_SKIS = "skis";
    public static final String DARKNET_OBJECT_SNOWBOARD = "snowboard";
    public static final String DARKNET_OBJECT_SPORTS_BALL = "sports ball";
    public static final String DARKNET_OBJECT_KITE = "kite";
    public static final String DARKNET_OBJECT_BASEBALL_BAT = "baseball bat";
    public static final String DARKNET_OBJECT_BASEBALL_GLOVE = "baseball glove";
    public static final String DARKNET_OBJECT_SKATEBOARD = "skateboard";
    public static final String DARKNET_OBJECT_SURFBOARD = "surfboard";
    public static final String DARKNET_OBJECT_TENNIS_RACKET = "tennis racket";
    public static final String DARKNET_OBJECT_BOTTLE = "bottle";
    public static final String DARKNET_OBJECT_WINE_GLASS = "wine glass";
    public static final String DARKNET_OBJECT_CUP = "cup";
    public static final String DARKNET_OBJECT_FORK = "fork";
    public static final String DARKNET_OBJECT_KNIFE = "knife";
    public static final String DARKNET_OBJECT_SPOON = "spoon";
    public static final String DARKNET_OBJECT_BOWL = "bowl";
    public static final String DARKNET_OBJECT_BANANA = "banana";
    public static final String DARKNET_OBJECT_APPLE = "apple";
    public static final String DARKNET_OBJECT_SANDWICH = "sandwich";
    public static final String DARKNET_OBJECT_ORANGE = "orange";
    public static final String DARKNET_OBJECT_BROCCOLI = "broccoli";
    public static final String DARKNET_OBJECT_CARROT = "carrot";
    public static final String DARKNET_OBJECT_HOT_DOG = "hot dog";
    public static final String DARKNET_OBJECT_PIZZA = "pizza";
    public static final String DARKNET_OBJECT_DONUT = "donut";
    public static final String DARKNET_OBJECT_CAKE = "cake";
    public static final String DARKNET_OBJECT_CHAIR = "chair";
    public static final String DARKNET_OBJECT_COUCH = "couch";
    public static final String DARKNET_OBJECT_POTTED_PLANT = "potted plant";
    public static final String DARKNET_OBJECT_BED = "bed";
    public static final String DARKNET_OBJECT_DINING_TABLE = "dining table";
    public static final String DARKNET_OBJECT_TOILET = "toilet";
    public static final String DARKNET_OBJECT_TV = "tv";
    public static final String DARKNET_OBJECT_LAPTOP = "laptop";
    public static final String DARKNET_OBJECT_MOUSE = "mouse";
    public static final String DARKNET_OBJECT_REMOTE = "remote";
    public static final String DARKNET_OBJECT_KEYBOARD = "keyboard";
    public static final String DARKNET_OBJECT_CELL_PHONE = "cell phone";
    public static final String DARKNET_OBJECT_MICROWAVE = "microwave";
    public static final String DARKNET_OBJECT_OVEN = "oven";
    public static final String DARKNET_OBJECT_TOASTER = "toaster";
    public static final String DARKNET_OBJECT_SINK = "sink";
    public static final String DARKNET_OBJECT_REGRIGERATOR = "refrigerator";
    public static final String DARKNET_OBJECT_BOOK = "book";
    public static final String DARKNET_OBJECT_CLOCK = "clock";
    public static final String DARKNET_OBJECT_VASE = "vase";
    public static final String DARKNET_OBJECT_SCISSORS = "scissors";
    public static final String DARKNET_OBJECT_TEDDY_BEAR = "teddy bear";
    public static final String DARKNET_OBJECT_HAIR_DRIER = "hair drier";
    public static final String DARKNET_OBJECT_TOOTHBRUSH = "toothbrush";
}

