/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.detection.objects;


import com.ml.vision.autoface.detection.AgeDetection;
import com.ml.vision.autoface.detection.EmotionDetection;
import com.ml.vision.autoface.detection.FaceDetection;
import com.ml.vision.autoface.detection.GenderDetection;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_core.CV_32F;
import org.bytedeco.javacpp.opencv_dnn;
import static org.bytedeco.javacpp.opencv_dnn.blobFromImage;
import static org.bytedeco.javacpp.opencv_imgproc.FONT_HERSHEY_DUPLEX;
import static org.bytedeco.javacpp.opencv_imgproc.FONT_HERSHEY_SIMPLEX;
import static org.bytedeco.javacpp.opencv_imgproc.LINE_8;
import static org.bytedeco.javacpp.opencv_imgproc.getTextSize;
import static org.bytedeco.javacpp.opencv_imgproc.putText;
import static org.bytedeco.javacpp.opencv_imgproc.rectangle;

/**
 *
 * @author apple
 */
public class Yolov3MobileSSD extends Yolov3{
    
    private final int IN_WIDTH = 300;
    private final int IN_HEIGHT = 300;
    private final float WH_RATIO = (float)IN_WIDTH / IN_HEIGHT;
    private final double IN_SCALE_FACTOR = 0.007843;
    private final double MEAN_VAL = 127.5;
    private final double THRESHOLD = 0.2;

         
    public Yolov3MobileSSD( opencv_dnn.Net net, java.util.List<String> lstClassName ){
        
        this.net = net;
        this.lstClassName = lstClassName;
    }
    
    public Yolov3MobileSSD( opencv_dnn.Net net, java.util.List<String> lstClassName, boolean count ){
        
        this.net = net;
        this.lstClassName = lstClassName;
        this.count = count;
    }
    
    public Yolov3MobileSSD( opencv_dnn.Net net, java.util.List<String> lstClassName, boolean count, boolean demographic, 
            FaceDetection detection, GenderDetection genderDetector, AgeDetection ageDetector, EmotionDetection emotionDetector ){
        
        this.net = net;
        this.lstClassName = lstClassName;
        this.count = count;
        this.demographic = demographic;
        this.detection = detection;
        this.genderDetector = genderDetector;
        this.ageDetector = ageDetector;
        this.emotionDetector = emotionDetector;
    }
    
    @Override
    public void detect( opencv_core.Mat frame ){
        
        if( isCount() && tracker == null )
            tracker = new CentroidTracker( maxDisappeared, maxDistance );
        
        if( isDemographic() ){
            lstDemographic = new java.util.ArrayList<>();
        }
        
        if( lstDemographic != null )
            lstDemographic.clear( );
        
        try{
//            resize( frame, frame, new opencv_core.Size( IN_WIDTH, IN_HEIGHT ));
            opencv_core.Mat blob = blobFromImage( frame , IN_SCALE_FACTOR,
            new opencv_core.Size(IN_WIDTH, IN_HEIGHT),
            new opencv_core.Scalar(MEAN_VAL, MEAN_VAL, MEAN_VAL, 0), false, false, CV_32F );
            net.setInput(blob);
            opencv_core.Mat detections = net.forward( );

            int cols = frame.cols();
            int rows = frame.rows();
            detections = detections.reshape(1, (int)detections.total() / 7);
        
                
            for (int i = 0; i < detections.rows(); ++i) {

                FloatIndexer indexer  = detections.createIndexer();
                double confidence = indexer.get(0, i, 2);

                if (confidence > THRESHOLD) {
                    int classId = (int) indexer.getDouble(0, i, 1);
                     //System.out.println( "Detected classID=" + classId );
                    if( classId > classNames.length - 1 || classId < 0 ) continue;
                    String label = classNames[classId];
                    
                    if( this.lstClassName.contains(label) ){
                      
                        int left   = (int)( indexer.getDouble(0, i, 3) * cols );
                        int top    = (int)( indexer.getDouble(0, i, 4) * rows );
                        int right  = (int)( indexer.getDouble(0, i, 5) * cols );
                        int bottom = (int)( indexer.getDouble(0, i, 6) * rows );

                        // Draw rectangle around detected object.
                        opencv_core.Rect lr = new opencv_core.Rect( new opencv_core.Point( left, top ), 
                            new opencv_core.Point( right, bottom ));
                        
                        int[] baseLine = new int[1];
                        opencv_core.Size labelSize = getTextSize(label, FONT_HERSHEY_DUPLEX, 1.2, 1, baseLine);
                        // Draw background for label.
                        opencv_core.Rect bg = new opencv_core.Rect( 
                                new opencv_core.Point(left, top - labelSize.height()), 
                            new opencv_core.Point(left + labelSize.width(), top + baseLine[0]) );

                        if( isRectangleObject() ){
                            rectangle( frame, lr, new opencv_core.Scalar(0, 0, 255, 0), 4, LINE_8, 0);
                            rectangle( frame, bg, new opencv_core.Scalar(30, 30, 30,0), 4, LINE_8, 0 );
                            // Write class name and confidence.
                            putText( frame, "" + label, new opencv_core.Point(left, top),
                                    FONT_HERSHEY_SIMPLEX, 1.2, new opencv_core.Scalar(0, 0, 200, 0) );
                        }
                        
                        //System.out.println( "ID=" + classId + " Name=" + label );
                        if( tracker != null ){
                            // id>0 means that new object otherwise existing object
                            TrackableObject result = tracker.update( lr, label );
                            if( !result.isDemographDone() ){
                                if( isDemographic() && detection != null && label.equals(MOBILE_OBJECT_PERSON))
                                   if( demograph( frame, lr, detection ) ) {result.setDemographDone(true);} 
                            }
                        }
                    }
                }
            }
            
            if( tracker != null ){
                tracker.check();
                checkInreasement( );
            }
            
            //imwrite ("yolov3t.jpg", frame );
        
        }catch(Exception ex){ex.printStackTrace();}
    }

    @Override
    public void checkInreasement( ){
        try{
            if( lastCount.isEmpty() ){
                getCount( ).entrySet().forEach((entry) -> {
                    lastCount.put( entry.getKey(), entry.getValue() );
                    increasedCount.put( entry.getKey(), entry.getValue() );
                });
            }else{
                increasedCount.clear();
                
                getCount( ).entrySet().forEach((entry) -> {
                    int tmpCount = lastCount.getOrDefault( entry.getKey(), 0);
                    int increase = entry.getValue() - tmpCount;
                    if( increase > 0)
                        increasedCount.put( entry.getKey(), increase );    
                    
                    lastCount.replace( entry.getKey(), entry.getValue() );
                    
                });
                
            }
        }catch(Exception ex){ex.printStackTrace();}
    }
    
    @Override
    public boolean demograph( opencv_core.Mat frame, opencv_core.Rect rect, FaceDetection detection ){
        boolean result = false;
        
        try{
            opencv_core.Mat capturedFace = new opencv_core.Mat( frame, rect );
            java.util.List<opencv_core.Mat> capture = detection.detectFace( capturedFace, false, false, FaceDetection.ALGORITHM_CASCADE_CLASSIFIER, true );
            if( capture != null && !capture.isEmpty()){
                opencv_core.Mat firstFace = capture.get(0);

                String age = ageDetector.caffemodel( firstFace );
                String gender = genderDetector.caffemodel( firstFace ).toString();
                String emo = emotionDetector.caffemodel( firstFace );
                lstDemographic.add( new String[] {age, gender, emo} );
                System.out.println("Age="+age+", Gender=" + gender +", Emotion=" + emo);
                result = true;
            }
        }catch(Exception ex){}
        return result;
    }
    
//    public static void main( String args[] ){
//        opencv_core.Mat imgbgr1 = imread( "sample/cats.jpg" );
//        java.util.List<String> lstClassName = new java.util.ArrayList<>();
//        
//        lstClassName.add( MOBILE_OBJECT_PERSON );
//        lstClassName.add(MOBILE_OBJECT_BICYCLE);
//        lstClassName.add(MOBILE_OBJECT_BUS);
//        lstClassName.add(MOBILE_OBJECT_CAR);
//        lstClassName.add(MOBILE_OBJECT_MOTORBIKE);
//        lstClassName.add(MOBILE_OBJECT_TRAIN);
//        
//        Yolov3MobileSSD y3 = new Yolov3MobileSSD( null, lstClassName, true, true, null );
//        y3.detect( imgbgr1 );
//    }
    
        
    public static final String[] classNames = { "background",
            "aeroplane", "bicycle", "bird", "boat",
            "bottle", "bus", "car", "cat", "chair",
            "cow", "diningtable", "dog", "horse",
            "motorbike", "person", "pottedplant",
            "sheep", "sofa", "train", "tvmonitor" };
   public static final String MOBILE_OBJECT_AEROPLANE = "aeroplane";
   public static final String MOBILE_OBJECT_BACKGROUND = "background";
   public static final String MOBILE_OBJECT_BICYCLE = "bicycle";
   public static final String MOBILE_OBJECT_BIRD = "bird";
   public static final String MOBILE_OBJECT_BOAT = "boat";
   public static final String MOBILE_OBJECT_BOTTLE = "bottle";
   public static final String MOBILE_OBJECT_BUS = "bus";
   public static final String MOBILE_OBJECT_CAR = "car";
   public static final String MOBILE_OBJECT_CAT = "cat";
   public static final String MOBILE_OBJECT_CHAIR = "chair";
   public static final String MOBILE_OBJECT_COW = "cow";
   public static final String MOBILE_OBJECT_DININGTABLE = "diningtable";
   public static final String MOBILE_OBJECT_DOG = "dog";
   public static final String MOBILE_OBJECT_HORSE = "horse";
   public static final String MOBILE_OBJECT_MOTORBIKE = "motorbike";
   public static final String MOBILE_OBJECT_PERSON = "person";
   public static final String MOBILE_OBJECT_POTTEDPLANT = "pottedplant";
   public static final String MOBILE_OBJECT_SHEEP = "sheep";
   public static final String MOBILE_OBJECT_SOFT = "sofa";
   public static final String MOBILE_OBJECT_TRAIN = "train";
   public static final String MOBILE_OBJECT_TV_MONITOR = "tvmonitor";
}

