/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.recognition;



import com.ml.vision.autoface.database.SQLiteManager;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.bytedeco.javacv.OpenCVFrameConverter;

/**
 *
 * @author User
 */
public class Recognition {
    
    private SQLiteManager sqlManager;
    private NetLoader netLoader;
    
    
    public Recognition(){
        sqlManager = new SQLiteManager();
        netLoader = new NetLoader2622();
    }

    public NetLoader getNetLoader() {
        return netLoader;
    }
    
    
    public Object[] predission( BufferedImage face ){
        return predission( new OpenCVFrameConverter.ToIplImage().convertToMat( new Java2DFrameConverter( ).convert( face )),
                0, 0 );
    }  

    
    public Object[] predission( opencv_core.Mat input, int limit, int offset ){
        opencv_core.Mat inputFeature = netLoader.processFace( input ) ;
        return sqlManager.searchFaceV2( inputFeature, limit, offset);
    }

    public int save( String name, opencv_core.Mat imgbgr, String profilePath, String photoPath, int linkId ){
        return save( 0, name, imgbgr, profilePath, photoPath, linkId );
    }
    
    public int save( int id, String name, opencv_core.Mat imgbgr, String profilePath, String photoPath, int linkId ){
        int result = -1;
        
        try{
            opencv_core.Mat feature = netLoader.processFace( imgbgr );
            FloatIndexer indexer  = feature.createIndexer();
            float fArr[] = new float[ netLoader.getNetSize() ];
            indexer.get(0, fArr);
            byte data[] = encode ( fArr ) ;
            
            if( id > 0)
                result = sqlManager.savePhotoV2( id, name, data, profilePath, photoPath, linkId );
            else
                result = sqlManager.savePhotoV2( name, data, profilePath, photoPath, linkId );
            
            System.out.println("Result=" + result+", id=" + id+", name=" + name);
        }catch(Exception ex){ex.printStackTrace();}
        
        return result;
    }
    
    public void saveFromFile( java.io.File root ){
        try{
                        
            File[] imageFiles = listImages( root );
            if( imageFiles == null || imageFiles.length < 1 ){
                File[] imageDir = root.listFiles( );
                for( File dir : imageDir){
                    if ( dir.isDirectory()  ){
                        fill( listImages(  dir ) );
                    }
                }
            }else{
                fill( imageFiles );
            }
        }catch(Exception ex){ex.printStackTrace();}
    }
    
    public void fill( File[] imageFiles ){
        if( imageFiles == null ) return;
                    
        for (File image : imageFiles) {
            int label = Integer.parseInt( image.getName().split("\\_")[0] );
            opencv_core.Mat img =  imread( image.getAbsolutePath());
            String name = image.getName( ).substring(  image.getName().split("\\_")[0].length() + 1 );//  image.getName().split("\\_")[1] ;  
            name = name.substring( 0, name.length() - 4 ).replaceAll("_", " ");
            //save( 0, name, imgbgr, profilePath, photoPath, linkId );
            save( label+"-"+name, img, image.getAbsolutePath(), image.getAbsolutePath(), 0 );
        }
    }    
    
    private File[] listImages( File imageDir ) throws Exception{
        FilenameFilter imgFilter = new FilenameFilter() {
            @Override
            public boolean accept(File dir, String name) {
                name = name.toLowerCase();
                return name.endsWith(".jpg") || name.endsWith(".pgm") || name.endsWith(".png");
            }
        };
            
        return imageDir.listFiles( imgFilter );
    }
    
    public byte[] encode (float floatArray[]) {
        byte byteArray[] = new byte[floatArray.length*4]; 

        // wrap the byte array to the byte buffer 
        ByteBuffer byteBuf = ByteBuffer.wrap(byteArray); 

        // create a view of the byte buffer as a float buffer 
        FloatBuffer floatBuf = byteBuf.asFloatBuffer(); 

        // now put the float array to the float buffer, 
        // it is actually stored to the byte array 
        floatBuf.put (floatArray); 

        return byteArray; 
    } 

    public float[] decode (byte byteArray[]) { 
        float floatArray[] = new float[byteArray.length/4]; 

        // wrap the source byte array to the byte buffer 
        ByteBuffer byteBuf = ByteBuffer.wrap(byteArray); 

        // create a view of the byte buffer as a float buffer 
        FloatBuffer floatBuf = byteBuf.asFloatBuffer(); 

        // now get the data from the float buffer to the float array, 
        // it is actually retrieved from the byte array 
        floatBuf.get (floatArray); 

        return floatArray; 
    } 
    
    int count = 1;
    public static void main( String args[] ){
        Recognition dbr = new Recognition();
        //dbr.saveFromFile(new java.io.File("trained2"));
        //dbr.train();//20

        java.io.File file = new java.io.File("mto/test/6_Jan_Jan.jpg");
        opencv_core.Mat imgbgr = imread( file.getAbsolutePath() );
        dbr.save( "Jan Jan", imgbgr, file.getAbsolutePath(), file.getAbsolutePath(), 0 );
        
        /*
        for( int i=0; i < 15; i++ ){
            long startTime = System.currentTimeMillis();
            String result[] = dbr.predission( imread( "mto/10072918_oe_Htaik_Aung-r.jpg" ) );
            System.out.println("Loop Searching time=" + (System.currentTimeMillis() - startTime ) );
            System.out.println("Label=" + result[0]+", Level=" + result[1]+", Name=" + result[3]);
            
            //long startTime = System.currentTimeMillis();
            //String[] result = dbr.predission( imread( "mto/10072918_oe_Htaik_Aung-r.jpg" ) );
        }
        */
    }
    
}
