/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.server;


import com.ml.vision.autoface.database.SQLiteManager;
import com.ml.vision.autoface.recognition.NetLoader;
import com.ml.vision.autoface.recognition.NetLoader2622;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;

/**
 *
 * @author apple
 */
public class FaceSearch implements Runnable{
    
    public static final int PROCESS_LIMIT = 3000;
    public static final int THREAD_POOL_SIZE = 20;
    private boolean clientStop = false;
    protected boolean isStopped = false;
    protected Thread runningThread= null;
    protected ExecutorService threadPool;
    private SQLiteManager sqlManager;
    private opencv_core.Mat inputFeature;
    private FaceSearchListener listener;
    
    public void setListener( FaceSearchListener listener ){
        this.listener = listener;
    }
    public interface FaceSearchListener {
        public void onResult( Object result[] );
    }
    public FaceSearch( SQLiteManager sqlManager, opencv_core.Mat inputFeature ){
        this.sqlManager = sqlManager;
        this.inputFeature = inputFeature;
        threadPool = Executors.newFixedThreadPool( THREAD_POOL_SIZE );
    }
    
    private void register(int i){
        try{
            Worker worker = new Worker( sqlManager, inputFeature, PROCESS_LIMIT, PROCESS_LIMIT * i);
            worker.setListener((Object[] result) -> {
                if( !result[3].equals("Unknown") )
                    listener.onResult(result);
            });
            this.threadPool.execute(  worker );
        }catch(Exception ex){ ex.printStackTrace();}
    }
    
    @Override
    public void run(){
        synchronized(this){
            this.runningThread = Thread.currentThread();
        }
        for( int j = 0; j < 15; j++ ){
        for( int i=0; i < THREAD_POOL_SIZE; i++)
            register( i );
        startTime = System.currentTimeMillis();            
        try {
                this.runningThread.sleep( 1000 );
            } catch (Exception e) { e.printStackTrace();}
        }
        /*
        while(! isStopped() ){
            try {
                if( clientStop ){
                    clientStop = false;
                    register();
                }
                this.runningThread.sleep( 1000 );
            } catch (Exception e) {
                if(isStopped()) {
                    System.out.println("Clients Stopped.") ;
                    break;
                }
                throw new RuntimeException( "Error accepting client connection", e);
            }
        }
        */
        this.threadPool.shutdown();
        System.out.println("Clients Stopped.") ;
    }

    private synchronized boolean isStopped() {
        return this.isStopped;
    }

    public synchronized void stop(){
        this.isStopped = true;
    }
    
    static long startTime;
    public static void main(String args[] ){
        
        SQLiteManager sqlManager = new SQLiteManager();
        NetLoader netLoader = new NetLoader2622();
        opencv_core.Mat inputFeature = netLoader.processFace( imread( "D:/Projects/extracted/celeba_faces/254756_254756.jpg" ) ) ; //rtmp.jpg
        
        
        FaceSearch client = new FaceSearch( sqlManager, inputFeature );
        client.setListener((Object[] result) -> {
            System.out.println("Searching time=" + (System.currentTimeMillis() - startTime ) );
            System.out.println("Label=" + result[0]+", Distance=" + result[1]+", Name=" + result[3]);
        });
        
        new Thread( client ).start();
        System.out.println("Stopping Clients");
        client.stop();
    }
}

class Worker implements Runnable{
    private SQLiteManager sqlManager;
    private opencv_core.Mat inputFeature;
    private int limit;
    private int offset;
    private WorkerListener listener;
      
    public interface WorkerListener {
        public void onResult( Object result[] );
    }
    
    public Worker( SQLiteManager sqlManager, opencv_core.Mat inputFeature, int limit, int offset ){
        this.sqlManager = sqlManager;
        this.inputFeature = inputFeature;
        this.limit = limit;
        this.offset = offset;
    }
    
    @Override
    public void run(){
        long iStartTime = System.currentTimeMillis();
        Object result[] = sqlManager.searchFaceV2(inputFeature.clone(), limit, offset);
        //System.out.println("Internal Searching time=" + (System.currentTimeMillis() - iStartTime ) );
        listener.onResult(result);
    }
    
    public void setListener( WorkerListener listener ){
        this.listener = listener;
    }
}
