/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.server;


import com.ml.vision.autoface.detection.AgeDetection;
import com.ml.vision.autoface.detection.EmotionDetection;
import com.ml.vision.autoface.detection.GenderDetection;
import com.ml.vision.autoface.listener.ServerListener;
import com.ml.vision.autoface.recognition.Recognition;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author apple
 */
public class Server implements Runnable{
    
    private static java.util.Properties pro;
    private GenderDetection genderDetector;
    private AgeDetection ageDetector;
    private EmotionDetection emotionDetector;
    private Recognition dbRecognition;
    
    protected int          serverPort   = 8080;
    protected ServerSocket serverSocket = null;
    protected boolean      isStopped    = false;
    protected Thread       runningThread= null;
    protected ExecutorService threadPool;
    private ServerListener listener;
    
    public Server( int port){
        this.serverPort = port;
        threadPool = Executors.newFixedThreadPool( 5 );
    }
    
    public Server( java.util.Properties pro ){
        this.pro = pro;
        this.serverPort = Integer.parseInt(pro.getProperty( NetworkSettings.PORT, "9000" ));
        threadPool = Executors.newFixedThreadPool( Integer.parseInt(pro.getProperty( NetworkSettings.MAX_WORKER, "50" )) );
    }
    
    @Override
    public void run(){
        synchronized(this){
            this.runningThread = Thread.currentThread();
            listener.onStart();
        }
        
        genderDetector = new GenderDetection();
        ageDetector = new AgeDetection();
        emotionDetector = new EmotionDetection();
        
        try {
            dbRecognition = new Recognition( );
            System.out.println( "Recognition loaded on Server=" );
            
        } catch (Exception ex) {
            Logger.getLogger(Server.class.getName()).log(Level.SEVERE, null, ex);
        }
        

        listener.onReady();
        
        openServerSocket();
        while(! isStopped() ){
            Socket clientSocket = null;
            try {
                clientSocket = this.serverSocket.accept();
            } catch (IOException e) {
                if(isStopped()) {
                    System.out.println("Server Stopped.") ;
                    break;
                }
                throw new RuntimeException( "Error accepting client connection", e);
            }
            this.threadPool.execute( new WorkerRunnable( clientSocket, "Thread Pooled Server", 
                    genderDetector, ageDetector, 
                    emotionDetector, pro, dbRecognition) );
        }
        listener.onStop();
        this.threadPool.shutdown();
        System.out.println("Server Stopped.") ;
    }

    private synchronized boolean isStopped() {
        return this.isStopped;
    }

    public synchronized void stop(){
        this.isStopped = true;
        try {
            this.serverSocket.close();
        } catch (IOException e) {
            throw new RuntimeException("Error closing server", e);
        }
    }

    private void openServerSocket() {
        try {
            this.serverSocket = new ServerSocket(this.serverPort);
        } catch (IOException e) {
            throw new RuntimeException("Cannot open port 8080", e);
        }
    }
    
    public static void main(String args[] ){
        Server server = new Server(9000);
        server.setListener(new ServerListener() {
            @Override
            public void onStart() { }
            @Override
            public void onReady() { }
            @Override
            public void onStop() { }
        });
        new Thread(server).start();

        try {
            Thread.sleep( 1000 * 60 * 60 * 5);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println("Stopping Server");
        server.stop();
    }
    
    public void setListener( ServerListener listener){
        this.listener = listener;
    }
}
