/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.server;

import com.ml.vision.autoface.detection.AgeDetection;
import com.ml.vision.autoface.detection.EmotionDetection;
import com.ml.vision.autoface.detection.GenderDetection;
import com.ml.vision.autoface.recognition.Recognition;
import com.ml.vision.autoface.util.Constants;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import org.bytedeco.javacpp.indexer.FloatIndexer;
import org.bytedeco.javacpp.opencv_core;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.bytedeco.javacv.OpenCVFrameConverter;

/**
 *
 * @author apple
 */
public class WorkerRunnable implements Runnable{
    public int SO_TIMEOUT = 1000 * 60 * 3;
    private GenderDetection genderDetector;
    private AgeDetection ageDetector;
    private EmotionDetection emotionDetector;
    protected Socket clientSocket = null;
    protected String serverText   = null;
    private java.util.Properties pro;
    private Recognition dbRecognition;

    public WorkerRunnable( Socket clientSocket, String serverText, 
            GenderDetection genderDetector, AgeDetection ageDetector, 
            EmotionDetection emotionDetector, java.util.Properties pro,
            Recognition dbRecognition ) {
        this.clientSocket = clientSocket;
        this.serverText   = serverText;
        this.genderDetector = genderDetector;
        this.ageDetector = ageDetector;
        this.emotionDetector = emotionDetector;
        this.pro = pro;
        this.dbRecognition = dbRecognition;
        if( pro != null ){
            SO_TIMEOUT = Integer.parseInt( pro.getProperty( NetworkSettings.PORT, "180" )) * 1000;
        }
    }

    
        
    private void featureExtractor( DataInputStream dis, DataOutputStream dos ){
        
        try{
            java.io.File tmpFile = new java.io.File("tmp");
            if( !tmpFile.exists() || !tmpFile.isDirectory() ) 
                tmpFile.mkdir();
            
            String data = dis.readUTF( );
            String info[] = data.split( "," );
            int totalFiles = dis.readInt( );
            System.out.println( "Total request=" + totalFiles );
            dos.writeInt(dbRecognition.getNetLoader().getNetSize());
            dos.flush();
            
            for( int j = 1; j <= totalFiles; j++ ){
                
                long remaningByte = dis.readLong( );
                System.out.println("Total image bytes=" + remaningByte );
                ByteArrayOutputStream ous = null;
                byte[] buffer = new byte[4096];
                ous = new ByteArrayOutputStream( );
                int read = 0;
                while ( remaningByte>0 && 
                        ( read = dis.read(buffer,0, (int)Math.min( buffer.length, remaningByte )) ) != -1){
                    ous.write( buffer, 0, read );
                    remaningByte -= read;
                }
                ous.close();
                
                java.io.File file1 = new java.io.File( tmpFile, System.currentTimeMillis()+".jpg" );
                BufferedImage img = ImageIO.read( new ByteArrayInputStream( ous.toByteArray() ) );
                ImageIO.write( img, "jpg", file1 );
                opencv_core.Mat feature = dbRecognition.getNetLoader( ).processFace( imread( file1.getAbsolutePath() ) ) ;
                            
                FloatIndexer indexer  = feature.createIndexer();
                float fArr[] = new float[ dbRecognition.getNetLoader().getNetSize() ];
                indexer.get( 0, fArr );
                byte featureData[] = dbRecognition.encode ( fArr ) ;
                
                ByteArrayInputStream ios;   
                try {
                    byte[] buff = new byte[4096];
                    ios = new ByteArrayInputStream( featureData );
                    int r = 0;
                    
                    dos.writeLong(featureData.length);
                    dos.flush();
                    System.out.println("Byte length=" + featureData.length );
                    while ((r = ios.read(buff)) != -1) {
                        dos.write(buff, 0, r);
                        dos.flush( );
                    }
                    ios.close();
                    
                }catch (Exception ex){ex.printStackTrace();}
                
                boolean expression = dis.readBoolean();// expression, gender,age
                if( expression ){
                    String age = ageDetector.caffemodel(img);
                    String gender = genderDetector.caffemodel(img).toString();
                    String emo = emotionDetector.caffemodel(img);
                    dos.writeUTF(age+","+gender+","+emo);
                    dos.flush();
                }
                
                file1.delete( );
            }
           System.out.println("Extraction completed, total=" + totalFiles);
        }catch(Exception ex){
            ex.printStackTrace();
            try {
                dos.writeUTF( "Failed" );
                dos.flush();
            } catch (IOException ex1) {
                Logger.getLogger(WorkerRunnable.class.getName()).log(Level.SEVERE, null, ex1);
            }
        }
    }
     
    
        /**
     * command UTF = register, faceName UTF, fileLength = long, fileData = bytes,  
     * @param dis
     * @param dos 
     */
    private void register( DataInputStream dis, DataOutputStream dos ){
        
        try{
            java.io.File tmpFile = new java.io.File("tmp");
            if( !tmpFile.exists() || !tmpFile.isDirectory() )
                tmpFile.mkdir();
            
            int id = 0;
            int linkId = 0;
            String name = dis.readUTF( );
            
            if( name.contains("-")){
                id = Integer.parseInt(name.split("\\-")[0].trim());
                name = name.split("\\-")[1].trim();
                linkId = Integer.parseInt( name.split("\\-")[2].trim() );
            }
            
            long remaningByte = dis.readLong( );
            ByteArrayOutputStream ous = null;
            byte[] buffer = new byte[Constants.BUFFER_SIZE];
            ous = new ByteArrayOutputStream( );
            int read = 0;
            
            while ( remaningByte>0 && 
                    ( read = dis.read(buffer,0, (int)Math.min( buffer.length, remaningByte )) ) != -1){
                
                ous.write(buffer, 0, read);
                remaningByte -= read;
                System.out.println("Read=" + read+", Remaining=" + remaningByte);
            }
            ous.close();
            
            BufferedImage img = ImageIO.read( new ByteArrayInputStream( ous.toByteArray() ) );
            java.io.File file1 = new java.io.File( tmpFile, System.currentTimeMillis() + ".jpg" );
            ImageIO.write( img, "jpg", file1 );
                //dbr.save( "Jan Jan", imgbgr, file.getAbsolutePath(), file.getAbsolutePath(), 0 );
             opencv_core.Mat imgMat =  imread( file1.getAbsolutePath());
             if( id > 0)
                dbRecognition.save( id, name, imgMat, file1.getAbsolutePath(), file1.getAbsolutePath(), linkId );
             else
                 id = dbRecognition.save( name, imgMat, file1.getAbsolutePath(), file1.getAbsolutePath(), linkId );
            
            dos.writeUTF( id + "" );
            dos.flush();
            
            file1.delete();
        }catch(Exception ex){
            try {
                dos.writeUTF( "Failed" );
                dos.flush();
            } catch (IOException ex1) {
                Logger.getLogger(WorkerRunnable.class.getName()).log(Level.SEVERE, null, ex1);
            }
        }
    }
     
    @Override
    public void run() {
                
        try{
            clientSocket.setSoTimeout( SO_TIMEOUT );
            System.out.println("Incoming start*****");
            
            DataInputStream dis = new DataInputStream(new BufferedInputStream( clientSocket.getInputStream()));
            DataOutputStream dos = new DataOutputStream(new BufferedOutputStream( clientSocket.getOutputStream()));
            
            String command = dis.readUTF();
            System.out.println("UTF="+ command );
            
            if( command.equalsIgnoreCase("register")){
                register( dis, dos );
            }else if( command.equalsIgnoreCase("recognize")){
                recognize( dis, dos );
            }else if( command.equalsIgnoreCase("extract")){
                featureExtractor( dis, dos );
            }else if( command.equalsIgnoreCase("multirecognize")){
                multiRecognize( dis, dos );
            }
            dis.close();
            dos.close();
            clientSocket.close();
            
        }catch(Exception ex){ex.printStackTrace();}
    }
     
        /**
         * command UTF = recognize, ID UTF, fileLength = long, fileData = bytes 
         * @param dis
         * @param dos 
         */
    private void recognize( DataInputStream dis, DataOutputStream dos ){
            
        try{ 
            
            java.io.File tmpFile = new java.io.File("tmp");
            if( !tmpFile.exists() || !tmpFile.isDirectory() ) 
                tmpFile.mkdir();
            
            String id = dis.readUTF( );
            long remaningByte = dis.readLong( );// file size
            
            ByteArrayOutputStream ous = null;
            byte[] buffer = new byte[Constants.BUFFER_SIZE];
            ous = new ByteArrayOutputStream();
            int read = 0;
            
            while ( remaningByte>0 && 
                    ( read = dis.read(buffer,0, (int)Math.min( buffer.length, remaningByte )) ) != -1){
                
                ous.write(buffer, 0, read);
                remaningByte -= read;
            }
            ous.close();
            
            BufferedImage img = ImageIO.read( new ByteArrayInputStream( ous.toByteArray() ) );
            java.io.File file1 = new java.io.File( tmpFile, System.currentTimeMillis() + ".jpg" );
            ImageIO.write( img, "jpg", file1 );
            
            Object dataObj[] = dbRecognition.predission( img );
            String data [] = (String[]) dataObj[0];
            //{ _id +"", name, rDate+"", linkId+"", distance +"" };
            
            //TODO should be used json format
            if( data.length > 3 ){
                System.out.println( "ID=" + data[0]+", Name=" + data[1]+", Date=" + data[2] +", LinkId=" + data[3] +", Distance=" + data[4] );
                dos.writeUTF( ( data[0] ) + "," + data[1] + "," + data[2] + "," + data[3] + "," + data[4] + "," + Constants.VGG_ACCEPTANCE_LEVEL );
            }else{
                System.out.println( "ID=" + data[0]+", Name=" + data[1]+", Distance=" + data[2] );
                dos.writeUTF( ( data[0] ) + "," + data[1] + "," + data[2] + "," + Constants.VGG_ACCEPTANCE_LEVEL );
            }
            dos.flush( );
            
            file1.delete( );
            
        }catch( Exception ex ){
            ex.printStackTrace();
            try {
                dos.writeUTF( "Failed" );
                dos.flush();
            } catch (IOException ex1) {
                Logger.getLogger(WorkerRunnable.class.getName()).log(Level.SEVERE, null, ex1);
            }
        }
    }

    
    private void multiRecognize( DataInputStream dis, DataOutputStream dos ){
        
        try{         
            java.io.File tmpFile = new java.io.File("tmp");
            if( !tmpFile.exists() || !tmpFile.isDirectory() ) 
                tmpFile.mkdir();
            
            String data = dis.readUTF( );
            String info[] = data.split( "," );
            int totalFiles = dis.readInt( );
            System.out.println( "Total request=" + totalFiles );
            
            for( int j = 1; j <= totalFiles; j++ ){
                
                long remaningByte = dis.readLong( );
                System.out.println("Total image bytes=" + remaningByte );
                ByteArrayOutputStream ous = null;
                byte[] buffer = new byte[Constants.BUFFER_SIZE];
                ous = new ByteArrayOutputStream( );
                int read = 0;
                while ( remaningByte>0 && 
                        ( read = dis.read(buffer,0, (int)Math.min( buffer.length, remaningByte )) ) != -1){
                    ous.write( buffer, 0, read );
                    remaningByte -= read;
                }
                ous.close();
                
                java.io.File file1 = new java.io.File( tmpFile, System.currentTimeMillis()+".jpg" );
                BufferedImage img = ImageIO.read( new ByteArrayInputStream( ous.toByteArray() ) );
                ImageIO.write( img, "jpg", file1 );
                
                Object dataObj[] = dbRecognition.predission( img );
                String resultData [] = (String[]) dataObj[0];

                //TODO should be used json format
                if( resultData.length > 3 ){
                    System.out.println( "ID=" + resultData[0]+", Name=" + resultData[1]+", Date=" + resultData[2] +", LinkId=" + resultData[3] +", Distance=" + resultData[4] );
                    dos.writeUTF( ( resultData[0] ) + "," + resultData[1] + "," + resultData[2] + "," + resultData[3] + "," + resultData[4] + "," + Constants.VGG_ACCEPTANCE_LEVEL );
                }else{
                    System.out.println( "ID=" + resultData[0]+", Name=" + resultData[1]+", Distance=" + resultData[2] );
                    dos.writeUTF( ( resultData[0] ) + "," + resultData[1] + "," + resultData[2] + "," + Constants.VGG_ACCEPTANCE_LEVEL );
                }
                dos.flush( );
                
                
                if( dataObj[1] != null ){
                    dos.writeBoolean(true);
                    dos.flush( );
                    
                    byte featureData[] = (byte[]) dataObj[1];
                    ByteArrayInputStream ios;   
                    try {
                        byte[] buff = new byte[Constants.BUFFER_SIZE];
                        ios = new ByteArrayInputStream( featureData );
                        int r = 0;

                        dos.writeLong(featureData.length);
                        dos.flush();
                        System.out.println("Byte length=" + featureData.length );
                        while ((r = ios.read(buff)) != -1) {
                            dos.write(buff, 0, r);
                            dos.flush( );
                        }
                        ios.close();

                    }catch (Exception ex){ex.printStackTrace();}
                }else{
                    dos.writeBoolean( false );
                    dos.flush( );
                }
                
                boolean expression = dis.readBoolean();// expression, gender,age
                if( expression ){
                    String age = ageDetector.caffemodel(img);
                    String gender = genderDetector.caffemodel(img).toString();
                    String emo = emotionDetector.caffemodel(img);
                    dos.writeUTF(age+","+gender+","+emo);
                    dos.flush();
                }
                
                file1.delete( );
            }
           System.out.println("Extraction completed, total=" + totalFiles);
        }catch(Exception ex){
            ex.printStackTrace();
            try {
                dos.writeUTF( "Failed" );
                dos.flush();
            } catch (IOException ex1) {
                Logger.getLogger(WorkerRunnable.class.getName()).log(Level.SEVERE, null, ex1);
            }
        }
    }
     
}
