/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ml.vision.autoface.ui.ad.util;


import org.bytedeco.javacpp.opencv_core;
import org.bytedeco.javacpp.opencv_videoio;
import static org.bytedeco.javacpp.opencv_videoio.CAP_PROP_FPS;
import static org.bytedeco.javacpp.opencv_videoio.CAP_PROP_FRAME_HEIGHT;
import static org.bytedeco.javacpp.opencv_videoio.CAP_PROP_FRAME_WIDTH;
import static org.bytedeco.javacpp.opencv_videoio.CAP_PROP_POS_FRAMES;

/**
 *
 * @author apple
 */
public class VideoPlayer extends Thread implements Runnable {
    
    public static final String STATUS_READY = "ready";
    public static final String STATUS_PLAYING = "playing";
    public static final String STATUS_LOADING = "loading";
    public static final String STATUS_FINISHED = "finished";
    public static final String STATUS_FAILED = "failed";
    
    public interface VideoPlayerListener{
        public void onPlay( opencv_core.Mat colorImage );
        public void onStatus( String status );
    }
    
    private VideoPlayerListener listener;
    private opencv_videoio.VideoCapture capture;
    private boolean stop;
    private int fps = 3000;
    
    public void initialize( String fileName ){
        listener.onStatus(STATUS_LOADING);
        try{
            capture = new opencv_videoio.VideoCapture( fileName );
            capture.set(CAP_PROP_FRAME_WIDTH, 1080);
            capture.set(CAP_PROP_FRAME_HEIGHT, 720);
            fps =(int) capture.get(CAP_PROP_FPS);
            int width =(int) capture.get( CAP_PROP_FRAME_WIDTH );
            int height =(int) capture.get( CAP_PROP_FRAME_HEIGHT);
            
            if (!capture.open( fileName )) {
                listener.onStatus(STATUS_FAILED);
            }else{
                listener.onStatus(STATUS_READY);
            }
        }catch(Exception ex){
                listener.onStatus(STATUS_FAILED);
            ex.printStackTrace();
        }
    }
    

    @Override
    public void run(){
        
        try{
            int count = 0;
            Outer:while (capture.isOpened() && !isStop()) {
                listener.onStatus( STATUS_PLAYING );
                opencv_core.Mat colorimg = new opencv_core.Mat();
                while (capture.read(colorimg) ) {
                    listener.onPlay( colorimg.clone() );
                    //int currentFrame = ++count * fps;
                    //capture.set( CAP_PROP_POS_FRAMES, count++ );
                    System.out.println( "Current frame="+count+", FPS=" + fps );
                    
                    if( isStop()){
                        capture.close(); break;
                    }
                    Thread.sleep( Math.round( 1000 / fps) );
                    //sleep( 1000 / fps );
                }
                listener.onStatus( STATUS_FINISHED );
                setStop(true);
                if(isStop()){
                    capture.close(); break;
                }
            }
            
            listener.onStatus( STATUS_FINISHED );
            setStop(true);
        }catch(Exception ex){
            listener.onStatus(STATUS_FAILED);
            setStop(true);
            ex.printStackTrace();
        }
    }

    public boolean isStop() {
        return stop;
    }

    public void setStop(boolean stop) {
        this.stop = stop;
    }
    
    
    public void setListener( VideoPlayerListener listener ){
        this.listener = listener;
    }
}
